      subroutine islogw ( tauw  , uplus, physut, ndist, tke,
     +                    rintur, rhow , mu    , rough      )
c=======================================================================
c
c
c        programmer     Marcel Zijlema
c        version 1.3    date   02-04-97 Kees K. literal constants in
c                                       double precision for f90
c        version 1.2    date   08-12-95 Kees K. intinsic function names 
c        version 1.1    date   31-08-95 (Minor change w.r.t. iseropen/clos)
c        version 1.0    date   23-09-94
c
c
c
c   copyright (c) 1994-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c***********************************************************************
c
c                  DESCRIPTION
c
c     Compute the wall shear stress and wall-coordinate velocity (U^+)
c     based on the physical tangential velocity at distance ndist normal
c     to the wall with logarithmic or linear law of the wall. This
c     routine take account of the wall roughness
c
c***********************************************************************
c
c                  KEYWORDS
c
c     turbulence
c
c***********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
c
      double precision  physut, ndist, tke, rintur(*), mu, rough,
     +                  tauw, uplus, rhow
c
c     mu       i   The laminar viscosity
c     ndist    i   Distance normal to a solid wall
c     physut   i   The physical velocity tangent to the wall
c     rhow     i   Specific weight
c     rintur   i   reserved
c     rough    i   The average roughness height
c     tauw     o   The wall shear stress
c     tke      i   The turbulent kinetic energy
c     uplus    o   The dimensionless tangential velocity
c
c***********************************************************************
c
c                      COMMON BLOCKS
c
c***********************************************************************
c
c                      LOCAL PARAMETERS
c
      double precision  ev, hro, roure, yplus
c
c     ev        The edge of viscous sublayer where an abrupt
c               changeover from purely viscous to purely turbulent
c               transport mechanism takes place. It is determined
c               by the following implicit equation
c
c                     ev = 1/kappa * ln(E*ev)
c
c     hro       Roughness height ( help variable )
c     roure     The roughness Reynolds number based on the
c               average roughness height and velocity scale
c               (i.e. sqrt(tke))
c     yplus     The dimensionless normal distance to the wall
c
c***********************************************************************
c
c                      SUBROUTINES CALLED
c
c     ISERCLOS      removes name of subroutine from error list
c     ISEROPEN      adds name of subroutine to error list
c     ISERRSUB      gives error messages
c
c***********************************************************************
c
c                      I/O
c
c***********************************************************************
c
c                      ERROR MESSAGES
c
c   1200  Negative k is found -- run terminated!
c
c***********************************************************************
c
c                      PSEUDO CODE
c
c
c     Y+ = rho cmu^(1/4) sqrt(tke) Y / mu
c
c     roughness Reynolds number = rho cmu^(1/4) sqrt(tke) height / mu
c
c     if roughness Reynolds number < 11.3
c     then
c       wall is considered to be smooth, i.e. height = 0
c     end if
c
c     if roughness height <> 0 then
c
c wall is rough
c
c       if the location of first computational point is between
c       the roughness elements then Y = height
c
c       tau_w = rho cmu^1/4 kappa sqrt(tke) u_tan / ln(E_rough*Y/height)
c       U+    = ln( E_rough*Y/height )/kappa
c
c     else
c
c wall is smooth
c
c       if the computational point lies in the viscous sublayer then
c
c         tau_w = mu * u_tan / Y
c         U+    = Y+
c
c       else if the point lies in the logarithmic layer then
c
c         tau_w = rho cmu^1/4 kappa sqrt(tke) u_tan / ln(E Y+)
c         U+    = ln( E Y+ )/kappa
c
c       end if
c
c     end if
c
c=======================================================================
c
c     --- Check whether negative turbulent energy occur

      if ( tke .lt. 0d0 ) then
         call iseropen('islogw')
         call iserrsub( 1200, 0, 0, 0 )
         call iserclos('islogw')
      end if

      ev = 11.3d0
      yplus = rhow*( rintur(3)**(25d-2) )*sqrt(tke)*ndist/mu
      roure = rhow*( rintur(3)**(25d-2) )*sqrt(tke)*rough/mu

      if ( roure .le. ev ) then
        hro = 0d0
      else
        hro = rough
      end if

      if ( hro .ne. 0d0 ) then

        if ( ndist .le. hro ) ndist = hro

        tauw = rhow*( rintur(3)**(25d-2) )*rintur(1)*sqrt(tke)*
     +         physut/log( rintur(11)*ndist/hro )

        uplus = log( rintur(11)*ndist/hro )/rintur(1)

      else

        if ( yplus .le. ev ) then

          tauw = mu * physut / ndist
          uplus = yplus

        else

          tauw = rhow*( rintur(3)**(25d-2) )*rintur(1)*sqrt(tke)*
     +           physut/log( rintur(2)*yplus )
          uplus = log( rintur(2)*yplus )/rintur(1)

        end if

      end if

      end
