      subroutine isomegb2( iside, i1, i2, bndcon, coefs, coor, solut,
     +                     rintur, ni, nj, nvirtual, ncoefs, eddy,
     +                     invjac, viscorr, ibinfor )
c ======================================================================
c
c        programmer     Marcel Zijlema
c        version 1.1    date   12-09-1995 Marcel: parameter list of
c                                               iswllqn2 changed
c        version 1.0    date   06-09-1995 Extracted from isomegbc
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     Fill BNDCON for the specific dissipation rate (omega) equation
c     with or without viscous corrections in R2
c **********************************************************************
c
c                       KEYWORDS
c
c     2d
c     turbulence
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer iside, i1, i2, nvirtual, ni, nj, ncoefs,
     +        viscorr, ibinfor(1-nvirtual:*)
      double precision coor(*), rintur(*), invjac(*), solut(*),
     +                 coefs(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,
     +                       1:ncoefs),
     +                 eddy(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual),
     +                 bndcon(2,1-nvirtual:*)

c     bndcon    o    reserved
c     coefs     i    reserved
c     coor      i    reserved
c     eddy      i    reserved
c     i1        i    Lower index number along first direction of a face or edge
c     i2        i    Upper index number along first direction of a face or edge
c     ibinfor   o    Third part of infbnd
c     invjac    i    1/sqrt(g) in pnttyp
c     iside     i    Sequence number of side
c     ncoefs    i    Number of coefficients
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     rintur    i    reserved
c     solut     i    reserved
c     viscorr   i    Indicate whether to use viscous corrections or not
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ip, jp, inci, incj, i, ndir, tdir, nsign
      double precision ndist, physut, nomeg, rhow, mu, yplus

c     i         General loop variable
c     inci      Offset between vertices in i-direction
c     incj      Offset between vertices in j-direction
c     ip        Local counting variable i-direction
c     jp        Local counting variable j-direction
c     mu        The laminar viscosity
c     ndir      Direction of normal vector on this side
c     ndist     Distance normal to a solid wall
c     nomeg     A constant to determine the analytical solution of
c               omega near a solid wall (in viscous sublayer)
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     physut    The physical velocity tangent to the wall
c     rhow      Specific weight
c     tdir      Direction of tangent vector on this side
c     yplus     The dimensionless normal distance to the wall
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  auxiliary subroutine to iserrsub
c     ISERREAL  auxiliary subroutine to iserrsub
c     ISERRSUB  gives error messages
c     ISWLLQN2  Determine essential wall quantities
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c   1201  Y+ > 5, which means that the grid near the wall is not fine
c         enough. You must either increase a number of grid points normal
c         to the wall or the cell aspect ratio
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
      call iseropen('isomegb2')

      if ( iside.le.2 ) then
         ndir = 1
         tdir = 2
         jp = i1-1
         inci  = 0
         incj  = 1
         if ( iside.eq.1 ) then
            nsign = -1
            ip = 1
         else
            ip = ni
            nsign = 1
         end if
      else
         ndir = 2
         tdir = 1
         ip = i1-1
         inci  = 1
         incj  = 0
         if ( iside.eq.3 ) then
            jp = 1
            nsign = -1
         else
            jp = nj
            nsign = 1
         end if
      end if

c     --- specific dissipation rate equation; fill BNDCON with or
c         without viscous corrections

      if ( viscorr.eq.0 ) then
        nomeg = 6d0/rintur(17)
      else if ( viscorr.eq.1 ) then
        nomeg = 2d0/rintur(16)
      end if

      do i = i1, i2-1

         ip = min(ip+inci,ni)
         jp = min(jp+incj,nj)

c        --- First, compute two essential wall quantities: distance
c            perpendicular to the wall and physical tangential velocity

         call iswllqn2 ( ndist, physut, ni, nj, nvirtual, ip, jp,
     +                   coor, invjac, solut, ndir, tdir, nsign, 1 )

         rhow = coefs(ip,jp,1)
         mu = coefs(ip,jp,4)-rhow*eddy(ip,jp)*rintur(19)
         yplus = sqrt(rhow*ndist*abs(physut)/mu)

         if ( yplus.gt.5d0 ) then
           call iserrchr( 'Y+', 1 )
           call iserreal( yplus, 1 )
           call iserrchr( 'Y+', 2 )
           call iserrsub ( 1201, 0, 1, 2 )
         end if

         bndcon(1,2*i-1) = nomeg*mu/(rhow*ndist*ndist)
         bndcon(1,2*i)  = bndcon(1,2*i-1)

      end do
      bndcon(1,2*i2-1) = bndcon(1,2*i2-2)

      do i = 2*i1-1, 2*i2-1
         ibinfor(i) = 3
      end do

      call iserclos('isomegb2')

      end
