      subroutine isbupwvel ( ni, nj, nvirtual, alphad, intp,
     +                       ibinfor, ibipoint, vectorupw )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 30-03-1999
c
c   copyright (c) 1999-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c **********************************************************************
c
c                       DESCRIPTION
c
c    Update the multiplication vector which indicates from which side upwind
c    must be taken for half cells where we have an outflow boundary condition
c **********************************************************************
c
c                       KEYWORDS
c
c    boundary_conditions
c    convection
c    matrix
c    right_hand_side
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, intp,
     +        alphad, ibipoint(*), ibinfor(*)
      double precision vectorupw(1-nvirtual:ni+1+nvirtual,
     +                           1-nvirtual:nj+1+nvirtual,2)

c     alphad     i   Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     ibinfor    i   Third part of infbnd
c     ibipoint   i   ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     intp       i   Interpolated values of variable
c     ni         i   Number of non-virtual cells in the i-direction (one block)
c     nj         i   Number of non-virtual cells in the j-direction (one block)
c     nvirtual   i   Number of rows of virtual cells
c     vectorupw i/o  Array containing the multiplication factors for upwind
c                    vector contains two positions for these factors
c                    The multiplication factor is a vector containing either:
c                    (1,0), (0,1) or (0.5,0.5)
c                    (1,0) means take point left or below integration point
c                    (0,1) means take point right or above integration point
c                    (0.5,0.5) means take mean value (i.e. central differences)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer snnsnt, snnut, parabolic, unut, uxuy, unsnt, walllaw
      parameter( unut = 4, uxuy = 10, unsnt = 5,
     +           walllaw = 9, parabolic= 12, snnsnt = 7, snnut = 6 )
      integer xp(2), ip, jp, btype, iside, tdir, ishift, jshift,
     +        ipibinf, ilow, jlow, iupp, jupp, betad
      logical ungiven

c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     btype     Type of boundary condition
c     ilow      Lower index of subface
c     ip        Local counting variable i-direction
c     ipibinf   Starting address of array infor for present side minus one
c     ishift    Shift in i-direction
c     iside     Sequence number of side
c     iupp      Upper i-index of subface
c     jlow      Lower j-index for a subface
c     jp        Local counting variable j-direction
c     jshift    Shift in j-direction
c     jupp      Upper j-index of subface
c     parabolic Parabolic boundary conditions
c     snnsnt    Sigma_nn and sigma_nt prescribed
c     snnut     Sigma_nn and u_t prescribed
c     tdir      Direction of tangent vector on this side
c     ungiven   If true un is given
c     unsnt     U_n and sigma_nt prescribed
c     unut      U_n and u_t prescribed
c     uxuy      U_x and u_y prescribed
c     walllaw   Law of the wall boundary condition
c     xp        Auxiliary array to calculate boundary point number
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c     Consider a half cell for example at the top boundary:
c
c
c      ---------------|--------------
c      |             (0,0)           |
c      |                             |
c      |                             |
c      |                             |
c      ------------------------------
c
c     The integration over the half cell means that the contribution of
c     the convective term is moved from (0,1) to (0,0)
c     With respect to the value in vsum, the value in (0,2) is made equal to
c     zero in vsum not in point (0,1), since for a conservative scheme
c     values are evaluated in the V^alpha points
c     As a consequence if the multiplication factor in (0,1) is not zero
c     the contribution of that point is added twice to (0,0)
c
c     For that reason we must set vectorupw equal to 0 for the point (0,1)
c     That is what is performed in this subroutine
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Loop over all sides

      do iside = 2*alphad-1, 2*alphad

         ilow = 1
         iupp = 1
         jlow = 1
         jupp = 1
         ipibinf = ibipoint(iside)-1+nvirtual
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            jupp = nj
            tdir = 2
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               ilow = ni+1
               iupp = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iupp = ni
            tdir = 1
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               jlow = nj+1
               jupp = nj+1

            end if

         end if
         ishift = 0
         jshift = 0
         if ( intp.eq.1 ) then

c        --- Points (-1,0) and (1,0) at boundaries 1 and 2
c            Points (0,-1) and (0,1) at boundaries 3 and 4
c            Point outside the region, set vsum = 0

            if ( iside.eq.1 ) then
               ilow = 1
               iupp = 1
               ishift = -1
            else if ( iside.eq.3 ) then
               jlow = 1
               jupp = 1
               jshift = -1
            end if
            if ( iside.eq.2 ) then
               ilow = ni+1
               iupp = ni+1
            else if ( iside.eq.4 ) then
               jlow = nj+1
               jupp = nj+1
            end if

            do jp = jlow, jupp
               do ip =  ilow, iupp
                  xp(1) = ip
                  xp(2) = jp
                  btype = ibinfor(ipibinf+2*xp(tdir))
                  if ( btype.eq.snnut .or. btype.eq.snnsnt ) then

c                 --- Half cell with given normal stress component
c                     Point outside the region, set vectorupw = 0

                     vectorupw(ip+ishift,jp+jshift,1) = 0d0
                     vectorupw(ip+ishift,jp+jshift,2) = 0d0

                  end if

               end do

            end do

         end if

      end do

c     --- Loop over all "tangential" sides

      betad = 3-alphad
      do iside = 2*betad-1, 2*betad

         ilow = 1
         iupp = 1
         jlow = 1
         jupp = 1
         ipibinf = ibipoint(iside)-1+nvirtual
         if ( iside.le.2 ) then

c        --- iside = 1, 2:  left- and right-hand side

            jupp = nj
            tdir = 2
            if ( iside.eq.2 ) then

c           --- iside = 2:  right-hand side

               ilow = ni+1
               iupp = ni+1

            end if

         else

c        --- iside = 3, 4:  lower- and upper side

            iupp = ni
            tdir = 1
            if ( iside.eq.4 ) then

c           --- iside = 4:  upper side

               jlow = nj+1
               jupp = nj+1

            end if

         end if
         if ( intp.eq.3 .and. iside.eq.2*betad-1 .or.
     +        intp.eq.4 .and. iside.eq.2*betad ) then

c        --- Point at boundary

            if ( iside.eq.1 ) then
               ilow = 1
               iupp = 1
            else if ( iside.eq.3 ) then
               jlow = 1
               jupp = 1
            end if
            if ( iside.eq.2 ) then
               ilow = ni+1
               iupp = ni+1
            else if ( iside.eq.4 ) then
               jlow = nj+1
               jupp = nj+1
            end if

            do jp = jlow, jupp
               do ip =  ilow, iupp
                  xp(1) = ip
                  xp(2) = jp
                  btype = ibinfor(ipibinf+2*xp(tdir))
                  ungiven = btype.eq.parabolic .or. btype.eq.88 .or.
     +                      btype.eq.unut .or. btype.eq.uxuy .or.
     +                      btype.eq.unsnt .or. btype.eq.walllaw
                  if ( ungiven ) then

c                 --- Point at boundary with un given, use central
c                     discretization

                     vectorupw(ip,jp,1) = 0.5d0
                     vectorupw(ip,jp,2) = 0.5d0

                  end if

               end do

            end do

         end if

      end do

      end
