      subroutine ischrys2 ( cva, jcta, chsym, invjac, ni, nj, nvirtual,
     +                      pnttyp )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.1    date 21-01-1998 Correction under and upper bounds
c        version 1.0    date 24-04-1995
c
c
c   copyright (c) 1995-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute the chrystoffel symbols in integration point
c    two-dimensional case
c **********************************************************************
c
c                       KEYWORDS
c
c    base_vectors
c    interpolation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, pnttyp
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4)
      double precision jcta(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2,2)
      double precision chsym(1-nvirtual:ni+1+nvirtual,
     +                       1-nvirtual:nj+1+nvirtual,2,3),
     +                 invjac(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual)

c     chsym     o    Christoffel symbols of the second kind in the
c                    point given by pnttyp
c                    (symmetric contravariant part stored as vector
c                    with index encoding given by istwo)
c     cva       i    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     invjac    i    1/sqrt(g) in pnttyp
c     jcta      i    Jacobian times contravariant basis vectors
c                    in point pnttyp
c                    (stored as vector index encoding given by istwo)
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pnttyp    i    reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, difpnttyp(4,2), lpnttyp, alphad, betad, gammad,
     +        istwo, dxp(2), dyp(2), d(4,2,2), ilowloop, iupploop,
     +        jlowloop, jupploop

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     betad     Direction index, usually the direction perpendicular to the
c               integration direction of the boundary integral.
c               For example betad in local point (1,0) is 1 and in (0,1): 2
c     d         Array with shifts for difference formulas
c               d(pnttype,i,betad) is shift in point of type point type
c               in betad direction
c               i = 1 refers to point with + sign
c               i = 2 refers to point with - sign
c     difpnttyp Contains the point types for the difference formulas
c               difpnttyp(pnttype,betad) is shift in point of type point type
c               in betad direction
c     dxp       auxiliary array
c     dyp       auxiliary array
c     gammad    Direction index
c     i         General loop variable
c     ilowloop  Lower bound for loop
c     istwo     Translation table to store and retrieve the
c               elements of a symmetric ndim-dimensional
c               rank 2 tensor from a vector with length
c               (ndim*(ndim+1))/2
c     iupploop  Upper bound for loop
c     j         General loop variable
c     jlowloop  Lower bound for loop
c     jupploop  Upper bound for loop
c     lpnttyp   Local point type
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The Christoffel symbol in a point is defined by:
c
c
c     |    alpha   |     (alpha)               beta
c     |            | =  a       . da     / d ksi
c     | beta gamma |                (gamma)
c
c                                  (alpha)
c     Since jcta contains sqrt(g) a        it is necessary to divide by sqrt(g)
c
c     The difference depends on beta and pnttyp
c
c     pnttyp is defined according to the following scheme:
c
c
c                ksi^2
c                 ^     +----|----+
c                 |     |         |
c                       |         |
c                      --    *    --
c                     3 |   4     |
c                       |         |
c                       +----|----+   -> ksi^1
c                     1     2
c
c
c    This means that a difference in 1-direction means
c       For pnttyp 1: use pnttyp 2 (present cell - preceding cell)
c       For pnttyp 2: use pnttyp 1 (next cell - present cell)
c       For pnttyp 3: use pnttyp 4 (present cell - preceding cell)
c       For pnttyp 4: use pnttyp 3 (next cell - present cell)
c    and for a difference in 2-direction
c       For pnttyp 1: use pnttyp 3 (present cell - preceding cell)
c       For pnttyp 2: use pnttyp 4 (present cell - preceding cell)
c       For pnttyp 3: use pnttyp 1 (next cell - present cell)
c       For pnttyp 4: use pnttyp 2 (next cell - present cell)
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data difpnttyp / 2, 1, 4, 3,
     +                 3, 4, 1, 2 /
      data d / 0, 1, 0, 1,    -1, 0, -1, 0,
     +         0, 0, 1, 1,    -1, -1, 0, 0/
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      do betad = 1, 2
         lpnttyp = difpnttyp(pnttyp,betad)
         if ( betad.eq.1 ) then
            dxp(1) = d(pnttyp,1,betad)
            dxp(2) = d(pnttyp,2,betad)
            dyp(1) = 0
            dyp(2) = 0
         else
            dyp(1) = d(pnttyp,1,betad)
            dyp(2) = d(pnttyp,2,betad)
            dxp(1) = 0
            dxp(2) = 0
         end if
         do gammad = 1, betad
            istwo = betad-1+gammad
            do alphad = 1, 2
               jlowloop = 1-nvirtual-min(0,dyp(1),dyp(2))
               jupploop = nj+1+nvirtual-max(0,dyp(1),dyp(2))
               ilowloop = 1-nvirtual-min(0,dxp(1),dxp(2))
               iupploop = ni+1+nvirtual-max(0,dxp(1),dxp(2))
               do j = jlowloop, jupploop
                  do i = ilowloop, iupploop
                     chsym(i,j,alphad,istwo) = invjac(i,j)*
     +                 (jcta(i,j,1,alphad)*
     +                   (cva(i+dxp(1),j+dyp(1),1,gammad,lpnttyp)-
     +                    cva(i+dxp(2),j+dyp(2),1,gammad,lpnttyp))+
     +                  jcta(i,j,2,alphad)*
     +                   (cva(i+dxp(1),j+dyp(1),2,gammad,lpnttyp)-
     +                    cva(i+dxp(2),j+dyp(2),2,gammad,lpnttyp)))
                  end do
               end do
            end do
         end do
      end do

      end
