      subroutine iscova2d ( coor, cva, ni, nj, nvirtual )
c ======================================================================
c
c
c        programmer     Guus Segal
c        version 1.1    date 04-10-95 (Remove pf, rspv and cv)
c        version 1.0    date 20-04-95 (Subroutine extracted from iscva)
c
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c
c ********************************************************************
c
c                      DESCRIPTION
c
c    Compute covariant base vectors in all points of the grid
c    two-dimensional case
c ********************************************************************
c
c                      KEYWORDS
c
c    base_vectors
c    interpolation
c ********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,2)
      double precision cva(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual,2,2,4)

c     coor      i    reserved
c     cva       o    Array containing the covariant basis vectors
c                    in 2D we have
c                                                 l
c                    cva(i,j,l,alpha,p) contains a        in cell i,j in
c                                                 (alpha)       point type p
c                    in 3D it is cva(i,j,k,l,alpha,p) referring to cell i,j,k
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c ********************************************************************
c
c                      COMMON BLOCKS
c
c ********************************************************************
c
c                      LOCAL PARAMETERS

      integer i, j, alphad

c     alphad    Direction index of integration.
c               Hence if a V1-cell is used, alphad = 1 and so on
c     i         General loop variable
c     j         General loop variable
c ********************************************************************
c
c                      SUBROUTINES CALLED
c
c ********************************************************************
c
c                      I/O
c
c ********************************************************************
c
c                      ERROR MESSAGES
c
c ********************************************************************
c
c                      PSEUDO CODE
c
c     The covariant basis vectors are given by:
c      alpha                    beta
c     a       = dx       / d xsi       = cva ( i, j, alpha, beta, k )
c      (beta)     alpha
c
c     i,j refers to the cell index and k (pnttyp) to the position of the point
c     in a cell
c
c     To evaluate the base vectors a central difference formula is applied
c
c     This leads to:
c
c     pnttype                 d/dksi1                  d/dksi2
c
c        1            (x(i+1,j)-x(i-1,j))/2       (x(i,j+1)-x(i,j-1))/2
c        2             x(i+1,j)-x(i,j)            (x(i+1,j+1)-x(i+1,j-1))/4+
c                                                 (x(i,j+1)-x(i,j-1))/4
c        3         (x(i+1,j+1)-x(i-1,j+1))/4+      x(i,j+1)-x(i,j)
c                    (x(i+1,j)-x(i-1,j))/4
c        4         (x(i+1,j+1)-x(i,j+1))/2+      (x(i+1,j+1)-x(i+1,j))/2+
c                    (x(i+1,j)-x(i,j))/2           (x(i,j+1)-x(i,j))/2
c
c     The integration points for all internal cells as well as one row of
c     virtual cells are filled.
c     In order to compute the Christoffel symbols, which require derivatives of
c     covariant base functions some points in the second row of virtual cells
c     must be filled.
c     The following cases are distinguished:
c
c     Row j = 1-nvirtual:     Fill for i=2-nvirtual (1) ni+nvirtual points 3,4
c                             Fill for i=1-nvirtual point 4
c     Row j = nj+nvirtual:    Fill for i=2-nvirtual (1) ni+nvirtual points 1-4
c     Col i = 1-nvirtual:     Fill for j=2-nvirtual (1) nj+nvirtual points 2,4
c     Col i = ni+nvirtual:    Fill for j=2-nvirtual (1) nj+nvirtual points 1-4
c ======================================================================
c
c     --- pnttype = 1:     (x(i+1,j)-x(i-1,j))/2       (x(i,j+1)-x(i,j-1))/2

      do alphad = 1, 2
         do j = 2-nvirtual, nj+nvirtual
            do i = 2-nvirtual, ni+nvirtual
               cva(i,j,alphad,1,1) =
     +            0.5d0*(coor(i+1,j,alphad)-coor(i-1,j,alphad))
            end do
         end do

         do j = 2-nvirtual, nj+nvirtual
            do i = 2-nvirtual, ni+nvirtual
               cva(i,j,alphad,2,1) =
     +            0.5d0*(coor(i,j+1,alphad)-coor(i,j-1,alphad))
            end do
         end do
      end do

c     --- pnttype = 2:    x(i+1,j)-x(i,j)
c                        (x(i+1,j+1)-x(i+1,j-1))/4+(x(i,j+1)-x(i,j-1))/4

      do alphad = 1, 2
         do j = 2-nvirtual, nj+nvirtual
            do i = 1-nvirtual, ni+nvirtual
               cva(i,j,alphad,1,2) =
     +            coor(i+1,j,alphad)-coor(i,j,alphad)
            end do
         end do

         do j = 2-nvirtual, nj+nvirtual
            do i = 1-nvirtual, ni+nvirtual
               cva(i,j,alphad,2,2) =
     +            0.25d0*(coor(i+1,j+1,alphad)-coor(i+1,j-1,alphad)+
     +                    coor(i  ,j+1,alphad)-coor(i  ,j-1,alphad))
            end do
         end do
      end do

c     --- pnttype = 3:  (x(i+1,j+1)-x(i-1,j+1))/4+(x(i+1,j)-x(i-1,j))/4
c                        x(i,j+1)-x(i,j)

      do alphad = 1, 2
         do j = 1-nvirtual, nj+nvirtual
            do i = 2-nvirtual, ni+nvirtual
               cva(i,j,alphad,1,3) =
     +            0.25d0*(coor(i+1,j+1,alphad)-coor(i-1,j+1,alphad)+
     +                    coor(i+1,j  ,alphad)-coor(i-1,j  ,alphad))
            end do
         end do

         do j = 1-nvirtual, nj+nvirtual
            do i = 2-nvirtual, ni+nvirtual
               cva(i,j,alphad,2,3) =
     +            coor(i,j+1,alphad)-coor(i,j,alphad)
            end do
         end do
      end do

c     --- pnttype = 4:  (x(i+1,j+1)-x(i,j+1))/2+(x(i+1,j)-x(i,j))/2
c                       (x(i+1,j+1)-x(i+1,j))/2+(x(i,j+1)-x(i,j))/2


      do alphad = 1, 2
         do j = 1-nvirtual, nj+nvirtual
            do i = 1-nvirtual, ni+nvirtual
               cva(i,j,alphad,1,4) =
     +            0.5d0*(coor(i+1,j+1,alphad)-coor(i,j+1,alphad)+
     +                   coor(i+1,j  ,alphad)-coor(i,j  ,alphad))
            end do
         end do

         do j = 1-nvirtual, nj+nvirtual
            do i = 1-nvirtual, ni+nvirtual
               cva(i,j,alphad,2,4) =
     +            0.5d0*(coor(i+1,j+1,alphad)-coor(i+1,j,alphad)+
     +                   coor(i  ,j+1,alphad)-coor(i  ,j,alphad))
           end do
         end do
      end do

      end
