      subroutine isgmatw3d ( ni, nj, nk, nvirtual, gmat, adir, jac,
     +                       ibipoint, ibindex, ibinfor, alphad,
     +                       xcentr )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 21-04-1998
c
c   copyright (c) 1998-1998  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Compute gradient matrix in R^3
c    WesBeek discretization
c **********************************************************************
c
c                       KEYWORDS
c
c    grad_matrix
c    matrix
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual, ibipoint(*), ibindex(2,2,*),
     +        ibinfor(*), alphad
      double precision gmat(*), jac(*), adir(*), xcentr(*)

c     adir      i    Contravariant base vector in integration cell center
c     alphad    i    Direction index of integration.
c                    Hence if a V1-cell is used, alphad = 1 and so on
c     gmat      o    Gradient matrix
c     ibindex   i    ibindex(1,1,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D)
c                    ibindex(2,1,side) last point index where information
c                                    is available on this side in
c                                    direction tdir (2D) or tdir(1) (3D
c                    ibindex(1,2,side)
c                                    first point index where information
c                                    is available on this side in
c                                    direction tdir or tdir(2) (3D)
c                    ibindex(2,2,side)
c                                    last point index where information
c                                    is available on this side in
c     ibinfor   i    Third part of infbnd
c     ibipoint  i    ipoint(side)
c                                 position of start of ibinfor information
c                                 for SIDE in ibinfor
c     jac       i    sqrt(g) in pnttyp
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     xcentr    i    Coordinates of centroids of cells
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer iface, ilow1, ihigh1, ilow2, ihigh2, nbfaces(6,3),
     +        nbface, ilow1nb, ihigh1nb, ilow2nb, ihigh2nb

c     iface     Face number
c     ihigh1    First upper index of an array
c     ihigh1nb  First upper index of an array with respect to neigbour
c     ihigh2    Second upper index of an array
c     ihigh2nb  Second upper index of an array with respect to neigbour
c     ilow1     First lower index of an array
c     ilow1nb   First lower index of an array with respect to neigbour
c     ilow2     Second lower index of an array
c     ilow2nb   Second lower index of an array with respect to neigbour
c     nbface    Neighbouring face number
c     nbfaces   contains the neighbour face for a face and alpha
c               as follows:  neighbour face = nbfaces(face,alpha)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISGMATW01 Compute boundary part of gradient matrix in R^3 for one edge
c               WesBeek discretization
c     ISGMATW02 Compute boundary part of gradient matrix in R^3 for one face
c               WesBeek discretization
c     ISGMATW03 Compute boundary part of gradient matrix in R^3 for one edge
c               WesBeek discretization
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c    Fill gmat for all V1-unknowns
c    Fill gmat for all V2-unknowns
c    Correct gmat for boundary cells as function of the boundary conditions
c    Fold all virtual pressures inside the region
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data nbfaces / 0, 0, 6, 5, 3, 4,
     +               5, 6, 0, 0, 2, 1,
     +               4, 3, 1, 2, 0, 0 /

c     nbfaces contains the neighbour face for a face and alpha
c     as follows:  neighbour face = nbfaces(face,alpha)
c     The table is as follows:
c     face     alpha1  nbface1     alpha2 nbface2
c       1         2       5          3       4
c       2         2       6          3       3
c       3         3       1          1       6
c       4         3       2          1       5
c       5         1       3          2       2
c       6         1       4          2       1
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Fill gmat for V^alpha unknowns (internal points)

      call isgmatw01 ( ni, nj, nk, nvirtual, gmat, xcentr, jac,
     +                 alphad, adir )

c     --- Correct gmat for boundary cells as function of the boundary conditions
c         Restriction to all faces (only the tangential unknowns)
c         The following combinations of tangential cells and face numbers
c         are possible:
c         faces 1,2:  alphad = 3,4,5,6
c         faces 3,4:  alphad = 5,6,1,2
c         faces 5,6:  alphad = 1,2,3,4

      do iface = 1, 6

         if ( (iface+1)/2.ne.alphad ) then

c        --- Tangential cell

            ilow1  = ibindex(1,1,iface)
            ihigh1 = ibindex(2,1,iface)
            ilow2  = ibindex(1,2,iface)
            ihigh2 = ibindex(2,2,iface)
            call isgmatw02 ( ni, nj, nk, nvirtual, gmat, xcentr, jac,
     +                       alphad, adir, iface,
     +                       ibinfor(ibipoint(iface)), ilow1,
     +                       ihigh1, ilow2, ihigh2 )

         end if

      end do

c     --- Finally correct for edges, where we have two one-sided differences
c         The neighbour faces are stores in array nbfaces as function of alphad

      do iface = 1, 6

         if ( (iface+1)/2.ne.alphad ) then

c        --- Tangential cell

            ilow1  = ibindex(1,1,iface)
            ihigh1 = ibindex(2,1,iface)
            ilow2  = ibindex(1,2,iface)
            ihigh2 = ibindex(2,2,iface)
            nbface = nbfaces(iface,alphad)
            ilow1nb  = ibindex(1,1,nbface)
            ihigh1nb = ibindex(2,1,nbface)
            ilow2nb  = ibindex(1,2,nbface)
            ihigh2nb = ibindex(2,2,nbface)
            call isgmatw03 ( ni, nj, nk, nvirtual, gmat, xcentr, jac,
     +                       alphad, adir, iface,
     +                       ibinfor(ibipoint(iface)), ilow1,
     +                       ihigh1, ilow2, ihigh2,
     +                       ibinfor(ibipoint(nbface)), ilow1nb,
     +                       ihigh1nb, ilow2nb, ihigh2nb )

         end if

      end do

      end
