      subroutine ismomf3d ( iface, ni, nj, nk, ilow1, ihigh1, ilow2,
     +                      ihigh2, mct, matrix, rhside, ibinfor,
     +                      bcgeo, bndcon )
c ======================================================================
c
c        programmer     Guus Segal
c        version 1.0    date 29-06-1995 subroutine based on ideas of ismmbf
c
c   copyright (c) 1995-1995  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                         DESCRIPTION
c
c    All parts of the matrix outside the region are folded to internal
c    domain for the momentum matrix in R3
c    The folding process depends on the types of boundary conditions
c **********************************************************************
c
c                      KEYWORDS
c
c **********************************************************************
c
c                      INPUT / OUTPUT    PARAMETERS
c
      implicit none
      integer ni, nj, nk, ilow1, ihigh1,ilow2, ihigh2,
     +        ibinfor(ilow1:ihigh1,ilow2:ihigh2), iface,
     +        mct(3,3,-2:2,-2:2,-2:2)
      double precision bcgeo(21,ilow1:ihigh1,ilow2:ihigh2),
     +                 rhside(ni+1,nj+1,nk+1,3),
     +                 matrix(ni+1,nj+1,nk+1,3,*),
     +                 bndcon(3,ilow1:ihigh1,ilow2:ihigh2)

c     bcgeo     i    reserved
c     bndcon    i    reserved
c     ibinfor   i    Third part of infbnd
c     iface     i    Face number
c     ihigh1    i    First upper index of an array
c     ihigh2    i    Second upper index of an array
c     ilow1     i    First lower index of an array
c     ilow2     i    Second lower index of an array
c     matrix    i    reserved
c     mct       i    Integer matrix information
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     rhside    i    reserved
c **********************************************************************
c
c                    COMMON BLOCKS
c
c **********************************************************************
c
c                    LOCAL PARAMETERS
c

      integer  ndir, tdir(2), nsign, iparea(2,3), nijk(3), ktdir

c     iparea    Area of integration points where a contribution to
c               the matrix is to be calculated
c     ktdir     Sequence number of tangential direction (1 or 2)
c     ndir      Direction of normal vector on this side
c     nijk      Vector of number of points in the i,j and k-directions
c     nsign     Inner product of outward normal in computational domain
c               with the base vector perpendicular to the face
c     tdir      Direction of tangent vector on this side
c **********************************************************************
c
c             SUBROUTINES CALLED
c
c     ISMOMF30  Tangential unknowns of tangential cell are folded
c               Subroutine is restricted to tangential unknowns in
c               the same direction as the cell
c     ISMOMF31  Tangential unknowns of tangential cell are folded
c               Subroutine is restricted to tangential unknowns in
c               the direction orthogonal to the cell
c     ISMOMF32  Normal unknowns of tangential cell are folded
c     ISMOMF33  Tangential unknowns of normal cells are folded
c     ISMOMF34  Normal unknowns of normal cells are folded
c
c **********************************************************************
c
c                              I/O
c
c **********************************************************************
c
c                    ERROR MESSAGES
c
c **********************************************************************
c
c                    METHOD
c
c    The folding process will be carried out per face.
c    Some rows corresponding to unknowns in the neighbourhood of the edges
c    and vertices require special attention
c
c    With respect to the boundary conditions the following combinations
c    are important:
c
c      n           (n)
c     V   = sgn(n.a   ) sqrtg sqrt(g^nn) u.n                       (1)
c               - -                      - -
c
c     If u^t is given:
c
c         we assume that both tangential velocities are zero,
c
c         u.t  = u.t  = 0
c         - -1   - -2
c
c         Hence, we have
c
c          t1      n
c         V   = - V  (g_t t  g_nt  - g_t t  g_nt )/det             (2)
c                        2 2     1      1 2     2
c
c          t2      n
c         V   = - V  (g_t t  g_nt  - g_t t  g_nt )/det
c                        1 1     2      1 2     1
c
c                                            2
c         with det = g_t t  g_t t  - (g_t t )
c                       1 1    2 2       1 2
c
c    u_n and u_t given
c    u_n given, u_t not
c    u_t given, u_n not
c    u_t nor u_n given
c    multiblock
c    pericodical boundary conditions
c
c    For illustration purposes we limit ourselves to the lower face and
c    in case of an edge to the combination of lower face and left-hand side face
c    The vertex is illustrated at the corner (0,0,0)
c    With respect to the tangential cells we limit ourselves to the cells
c    i=2 (1) ni, j=2(1)nj hence all tangential velocities not at other
c    boundaries
c
c    At the lower boundary (face 5, ndir=3, nsig=-1), we have two tangential
c    cells: a V1 (tdir=1) and a V2 (tdir=2) cell.
c    The only unknowns to be eliminated are those that are outside the
c    region. Unknowns at the boundary are kept.
c    For those cells the following unknowns are outside the region:
c
c                                V1            V2
c    normal unknowns (V3):    1,0,-3         0,1,-3
c                            -1,0,-3        0,-1,-3
c
c    tangential unknown V1:   0,-2,-2      -1,-1,-2
c                             0, 0,-2       1,-1,-2
c                             0, 2,-2      -1, 1,-2
c                            -2, 0,-2       1, 1,-2
c                             2, 0,-2
c    tangential unknown V2:  -1,-1,-2      -2, 0,-2
c                             1,-1,-2       0, 0,-2
c                            -1, 1,-2       2, 0,-2
c                             1, 1,-2       0,-2,-2
c                                           0, 2,-2
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n:
c            U^n(.,.,-3) = 2U^n(.,.,-1) - U^n(.,.,1)                   (3)
c    c. U^n and U^t given at the boundary (.,-1):
c            U^t(.,.,-2) = 2U^t(.,.,-1) - U^t(.,.,0)                   (4)
c    d. U^t not given:
c            U^t(.,.,-2) = 2U^t(.,.,0) - U^t(.,.,2)                    (5)
c    e. U^t given and U^n not:
c            At the point (0,0) use a combination of (2) and (4)
c            At the other points use (5)
c
c    At the lower boundary (face 5, ndir=3, nsig=-1), we have one normal
c    cell: a V3 (ndir=3) .
c    The only unknowns to be eliminated are those that are outside the
c    region. Unknowns at the boundary are kept.
c    For those cells the following unknowns are outside the region:
c
c                                 V3
c    5 normal unknowns  V3 :    0,-2,-2
c                               0, 0,-2
c                               0, 2,-2
c                              -2, 0,-2
c                               2, 0,-2
c
c    8 tangential unknowns V1: -3, 0,-1
c                               3, 0,-1
c                              -1, 0,-1
c                               1, 0,-1
c                              -1,-2,-1
c                               1,-2,-1
c                              -1, 2,-1
c                               1, 2,-1
c
c    8 tangential unknowns V2:  0,-3,-1
c                               0, 3,-1
c                               0,-1,-1
c                               0, 1,-1
c                              -2,-1,-1
c                              -2, 1,-1
c                               2,-1,-1
c                               2, 1,-1
c
c    The folding process is as follows:
c
c    a. Periodical and multiblock boundaries: no action should be taken
c    b. U^n:
c            U^n given: diagonal element 1, off-diagonal elements 0
c                       Value in right-hand side.
c            In all other situations we do:
c            U^n(.,.,-2) = 2U^n(.,.,0) - U^n(.,.,2)
c    d. U^t not given:
c            U^t(.,.,-1) = U^t(.,.,1)
c    e. U^t given:
c            use a combination of (2) and (4)
c            except for the points (-3,.,.),(3,.,.) and (.,-3,.),(.,3,.)
c            which must be folded first to other points outside
c            the region.
c
c
c                    PSEUDO CODE
c
c **********************************************************************
c
c                    DATA STATEMENTS
c
c ======================================================================
c
c     --- loop over faces

      nijk(1) = ni
      nijk(2) = nj
      nijk(3) = nk

c     --- Compute tdir and ndir

      if ( iface.le.2 ) then

c     --- iface = 1, 2:  back and front plane

         ndir    = 1
         tdir(1) = 2
         tdir(2) = 3
         if ( iface.eq.1 ) then

c        --- iface = 1:  back

            nsign = -1

         else

c        --- iface = 2:  front

            nsign = 1

         end if

      else if ( iface.gt.2  .and. iface.lt.5) then

c     --- iface = 3, 4:  left- and right plane

         ndir = 2
         tdir(1) = 3
         tdir(2) = 1
         if ( iface.eq.3 ) then

c        --- iface = 3:  left

            nsign = -1

         else

c        --- iface = 4:  right

            nsign = 1

         end if

      else

c     --- iface = 5, 6:  bottom and top

         ndir = 3
         tdir(1) = 1
         tdir(2) = 2

         if ( iface.eq.5 ) then

c        --- iface = 5:  bottom

            nsign = -1

         else

c        --- iface = 6:  top

            nsign = 1

         end if

      end if

c     --- loop over two sets of tangential cells
c         Compute loop bounds

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)
         iparea(2,ndir) = nijk(ndir)
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      do ktdir = 1, 2

         iparea(1,tdir(1)) = 1
         iparea(2,tdir(1)) = nijk(tdir(1))
         iparea(1,tdir(2)) = 1
         iparea(2,tdir(2)) = nijk(tdir(2))
         iparea(2,tdir(ktdir)) = iparea(2,tdir(ktdir))+1

c        --- Loop over five tangential unknowns outside the region
c            corresponding to the ktdir direction

         call ismomf30 ( ktdir, ndir, ni, nj, nk, iparea, ilow1,
     +                   ihigh1, ilow2, ihigh2, mct, matrix,
     +                   rhside, ibinfor, bcgeo, bndcon, nsign,
     +                   tdir )

c        --- Loop over four tangential unknowns outside the region
c            corresponding to the not ktdir direction

         call ismomf31 ( ktdir, ndir, ni, nj, nk, iparea, ilow1,
     +                   ihigh1, ilow2, ihigh2, mct, matrix,
     +                   rhside, ibinfor, bcgeo, bndcon, nsign,
     +                   tdir )

c        --- Loop over two normal unknowns outside the region

         call ismomf32 ( ktdir, ndir, ni, nj, nk, iparea, ilow1,
     +                   ihigh1, ilow2, ihigh2, mct, matrix,
     +                   rhside, ibinfor, bcgeo, bndcon, nsign,
     +                   tdir )

      end do

c     --- loop over normal half cells
c         Compute loop bounds

      if ( mod(iface,2).eq.0 ) then
         iparea(1,ndir) = nijk(ndir)+1
         iparea(2,ndir) = nijk(ndir)+1
      else
         iparea(1,ndir) = 1
         iparea(2,ndir) = 1
      end if

      iparea(2,tdir(1)) = nijk(tdir(1))
      iparea(2,tdir(2)) = nijk(tdir(2))

c     --- loop over eight tangential unknowns in both tangential
c         directions

      call ismomf33 ( ndir, ni, nj, nk, iparea, ilow1, ihigh1,
     +                ilow2, ihigh2, mct, matrix, ibinfor,
     +                bcgeo, nsign, tdir )

c     --- loop over five normal unknowns

      call ismomf34 ( ndir, ni, nj, nk, iparea, ilow1, ihigh1,
     +                ilow2, ihigh2, mct, matrix, rhside,
     +                ibinfor, bcgeo, bndcon, nsign, tdir )

      end
