      subroutine ispmat2d ( ni, nj, nvirtual, matrix, gmat, rinput,
     +                      iinput, jac, uold, vold, hnew,
     +                      shiftr, intp, massfrac, densup,
     +                      drhodp, coor, axisym, swsup )
c ======================================================================
c
c        programmer     Guus Segal
c        version 3.0    date 29-06-1999 Guus: new parameter list
c        version 2.3    date 27-03-1999 Duncan: Supersonic Cranck-Nicholson
c        version 2.2    date 25-03-1999 Duncan: extension for new
c                                               Supersonic Scheme
c        version 2.1    date 22-03-1999 Kees K: nondimensional compressible
c                                               standard pressure correction
c                                               no-upwind : axisymmetric
c        version 2.0    date 19-11-1998 Duncan: extension for multiblock
c        version 1.9    date 27-07-1998 Duncan: extension density
c                                               bias j direction
c        version 1.8    date 14-05-1998 Duncan: extension for density
c                                               upwind in new solution
c                                               method
c        version 1.7    date 01-04-1998 Duncan: New solution method
c                                               for solving pres eq.
c        version 1.6    date 05-03-1998 Duncan: Extension for cavity
c        version 1.5    date 11-12-1997 Hester: parameter mequat added
c        version 1.4    date 10-12-1997 Hester: parameter mprcor added
c        version 1.3    date 12-11-1997 Hester: parameter mscal added
c        version 1.2    date 15-05-1996 Hester: sign mistake corrected
c        version 1.1    date 26-02-1996 Hester: poss. of upwinded density
c                                                for compr. flow added.
c        version 1.0    date 06-10-1995
c
c   copyright (c) 1995-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c    Build pressure matrix in R2
c
c **********************************************************************
c
c                       KEYWORDS
c
c    2d
c    matrix
c    pressure_equation
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, iinput(*), axisym
      double precision jac(1-nvirtual:ni+1+nvirtual,
     +                     1-nvirtual:nj+1+nvirtual),
     +                 hnew(1-nvirtual:ni+1+nvirtual,
     +                   1-nvirtual:nj+1+nvirtual),
     +                 gmat(ni+1,nj+1,2,6),
     +                 matrix(ni+1,nj+1,9), rinput(*),
     +                 uold(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 shiftr(ni+1,nj+1,2),
     +                 vold(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual),
     +                 intp(1-nvirtual:ni+1+nvirtual,
     +                    1-nvirtual:nj+1+nvirtual,2),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 densup(ni+1,nj+1,3,2),
     +                 drhodp(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 coor(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual,1:2),
     +                 swsup(ni+1,nj+1,2)

c     axisym         i    In axi-symmetric case axisym=1 else axisym=0
c     coor           i    Array containing the coordinates for one block
c                         In 2d coor(i,j,.) contains the x resp y-coordinate
c                         of point i,j
c                         in 3d: coor(i,j,k,.) corresponds to (i,j,k)
c     densup        i/o   coefficients of density upwind in pressure matrix
c     drhodp         i    Contains drho/dp in all centres of the cells
c     gmat           i    Gradient matrix
c     hnew           i    Enthalpy at present time step
c     iinput         i    Contains the integer user input for one block.
c                         For a description see the programmers guide
c     intp           i    Interpolated values of variable
c     jac            i    sqrt(g) in pnttyp
c     massfrac       i    Array containing the mass fraction
c     matrix         o    Contains the matrix to be used in the solution process
c     ni             i    Number of non-virtual cells in the i-direction
c                         (one block)
c     nj             i    Number of non-virtual cells in the j-direction
c                         (one block)
c     nvirtual       i    Number of rows of virtual cells
c     rinput         i    The real user input is stored in the array rinput
c                         for each block
c                         For a description see the programmers guide
c     shiftr         i    Shift variable used in compressible density upwinding.
c     swsup          i    Switch for the supersonic scheme
c     uold           i    Velocity at previous time step.
c     vold           i    Velocity at previous time step
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, iprincom, mcom, mcomupw, mscal, mcav, acttyp,
     +        vartyp, pnttyp, mprcor, mequat, mcomsup, ichoicedb,
     +        ipiincom
      double precision deltatin, dt, gamcm, mach2, cons, cons1,
     +                 dummy(1), fact, facs

c     acttyp         Type of action to be taken
c                    0      : interpolation for point value
c                    1..ndim: finite difference for partial derivative
c                             in the direction given by value of acttyp
c     cons           Constant used in equation of state for compressible
c                    computations
c     cons1          Constant used in equation of state for compressible
c                    computations
c     deltatin       Inverse of deltat, i.e. 1/deltat
c     dt             Time step
c     dummy          Dummy
c     facs           Multiplication factor
c     fact           Multiplication factor
c     gamcm          Gas constant gamma (for compressible comput.)
c     i              General loop variable
c     ichoicedb      if 1 a gradient based, if 2 a mach based,
c                    if 3 an unconditional density bias is applied
c     ipiincom       Starting address of array IINCOM in array IINPUT
c     iprincom       Starting address of array RINCOM in array RINPUT
c     j              General loop variable
c     mach2          Square of the mach number (for compressible comput.)
c     mcav           If 1 a cavity model is used, if 0 not
c     mcom           If zero the flow is incompressible else (1)
c                    compressible
c     mcomsup        If 1 the supersonic scheme is activated if 0 not
c     mcomupw        If zero the rho equation for compressible flow is
c                    solved by a central scheme,
c                    otherwise by a upwind scheme
c     mequat         Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     mprcor         Indicates choice of compressible pressure correction
c                    method:
c                    0:   Original formulation
c                    1:   Pressure corr. equation better for contact
c                         discontinuities, eq. obtained from
c                         energy+momentum eq.
c     mscal          Indicates use of non-dimensionalisation for
c                    compr. flow:
c                    0:   Equations are non-dimensionalised
c                    1:   Equations are in dimensional formulation
c     pnttyp         Point where a value or finite difference is
c                    to be calculated within the cell
c                    For a description see the programmers guide
c     vartyp         Variable to be interpolated/extrapolated
c                    vartyp: 1 for V1, 2 for V2 and 3 for pressure,
c                    0 for vertex (for ndim=2)
c                    or
c                    vartyp: 1 for V1, 2 for V2 3, for V3 and
c                    4 for pressure, 0 for vertex  (for ndim=3)
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISINTCF2       Compute interpolation of a coefficient in an integration
c                    point
c     ISSETCON       Make array equal to a constant
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c
c    The pressure correction matrix has the shape:
c
c                 -1
c     theta dt D M   G
c
c     with:
c
c     D:        the divergence matrix
c     G:        the gradient matrix
c     M:        a diagonal matrix with rho at the main diagonal
c     theta dt: theta * time step
c
c
c     In case of a compressible flow the matrix is updated by extra terms
c     These terms depend on the method chosen.
c     Standard:
c
c      The diagonal is updated by 1/dt * d rho /dp
c
c
c
c
c     The pressure matrix has a 9-point stencil with the following numbering:
c
c
c
c                    *7        *8         *9
c
c
c
c                    *5        *1         *6
c
c
c
c                    *2        *3         *4
c
c     The divergence per cell looks like
c
c
c                          1
c                   -------|-------
c                   |             |
c                   |             |
c                -1 -             - 1
c                   |             |
c                   |             |
c                   -------|-------
c                         -1
c
c      The gradient matrix has the following shape
c
c                   1
c         -----------------------
c         |          |          |
c         |    5     |    6     |                          2
c         |    +     |    +     |
c         |          |          |          ----------------------------
c         ----------------------- |         |        |        |
c         |          |          |          |         |        |        |
c         |    3     _    4     |          |    +    |    +   |   +    |
c         |    +     |    +     |          |    2    |    4   |   6    |
c         |          |          |          ---------------|------------
c         ----------------------- |         |        |        |
c         |          |          |          |    +    |    +   |   +    |
c         |    1     |    2     |          |    1    |    3   |   5    |
c         |    +     |    +     |          |         |        |        |
c         |          |          |          ----------------------------
c         -----------------------
c
c
c    As a consequence the pressure matrix can be computed as follows:
c
c
c    1:  g(i+1,j,1,3)/rho(i+1,j) - g(i,j,1,4)/rho(i,j) +
c        g(i,j+1,2,3)/rho(i,j+1) - g(i,j,2,4)/rho(i,j)
c    2:  -g(i,j,1,1)/rho(i,j) - g(i,j,2,1)/rho(i,j)
c    3:  g(i+1,j,1,1)/rho(i+1,j) - g(i,j,1,2)/rho(i,j) +
c        -g(i,j,2,3)/rho(i,j)
c    4:  g(i+1,j,1,2)/rho(i+1,j) - g(i,j,2,5)/rho(i,j)
c    5:  g(i,j,1,3)/rho(i,j) +
c        g(i,j+1,2,1)/rho(i,j+1) - g(i,j,2,2)/rho(i,j)
c    6:  g(i+1,j,1,4)/rho(i,j) +
c        g(i,j+1,2,5)/rho(i,j+1) - g(i,j,2,6)/rho(i,j)
c    7:  -g(i,j,1,5)/rho(i,j) + g(i,j+1,2,2)/rho(i,j+1)
c    8:  g(i+1,j,1,5)/rho(i+1,j) - g(i,j,1,6)/rho(i,j) +
c        -g(i,j+1,2,4)/rho(i,j+1)
c    9:  g(i+1,j,1,6)/rho(i+1,j) + g(i,j+1,2,6)/rho(i,j+1)
c
c
c                    PSEUDO CODE
c
c
c     Clear pressure matrix
c     Evaluate 1/rho in integration points and multiply by theta dt
c     Multiply 1/rho by 2 for points corresponding to half cells
c     Construct the pressure matrix
c     If ( compressible flow ) then
c        Update diagonal
c     If the new Supersonic Scheme is chosen, the pressure correction matrix
c     will be formed in the following way
c
c
c    the continuity equation reads:
c      n+1     n                n+1            n  n+1
c   rho   - rho   + (1-sigma)( m  )   + sigma(u rho) = 0
c   ____________                   x                x
c
c        dt
c
c  where sigma is a Mach dependant switch. This means in the subsonic
c  case the compressible MAC scheme is used, which is able to
c  handle Mach ->0 , and in the supersonic case
c  the scheme transforms to an implicit convection scheme for rho, which is
c  known two work fine in the non-segregated approach.
c  This means the pressure correction equation will get the following form
c  when the u velocity has positive sign and the v velocity is 0
c
c    1                                      1       n    1         n
c  _____  dp   - (1-sigma)dt DG dp + sigma(___  dp u -  ___  dp   u  ) =
c    2,n    j                                2,n  j j+1   2,n  j-1 j
c   c  dt                                   c            c
c
c                   n  n       n  n
c         -sigma(rho  u -  rho   u  ) -(1-sigma)Dm*
c                   j  j+1    j-1 j
c
c this scheme is implemented as a form of density bias
c where the density bias coefficients take the simple form:
c
c  1    n            1    n
c ___  u     and    ___  u
c   2,n j+1           2,n j
c  c                 c
c
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
c     --- Clear pressure matrix

      call issetcon ( matrix, (ni+1)*(nj+1)*9, 0d0 )

      call issetcon ( intp, 2*(ni+1+2*nvirtual)*(nj+1+2*nvirtual),
     +                0d0 )

c     --- Construct the pressure matrix

      mcav      = iinput(77)

      ipiincom  = iinput(9)
      mcom      = iinput(ipiincom+1 - 1)
      mcomupw   = iinput(ipiincom+2 - 1)
      mequat    = iinput(ipiincom+3 - 1)
      mscal     = iinput(ipiincom+5 - 1)
      mprcor    = iinput(ipiincom+6 - 1)
      ichoicedb = iinput(ipiincom+7 - 1)

      if ( ichoicedb.ge.4)  then
         mcomsup = 1
      else
         mcomsup = 0
      end if

      if ( mcomupw.eq.0 ) then

c     --- no compressible density upwind, i.e. standard method

         if ( mprcor.eq.0 ) then

c        --- no compressible different pressure correction eq.,
c            i.e. standard method for the construction of the matrix

            do j = 1, nj
               do i = 1, ni

                  matrix(i,j,1) =  gmat(i+1,j  ,1,3)
     +                            -gmat(i  ,j  ,1,4)
     +                            +gmat(i  ,j+1,2,3)
     +                            -gmat(i  ,j  ,2,4)
                  matrix(i,j,2) = -gmat(i  ,j  ,1,1)
     +                            -gmat(i  ,j  ,2,1)
                  matrix(i,j,3) =  gmat(i+1,j  ,1,1)
     +                            -gmat(i  ,j  ,1,2)
     +                            -gmat(i  ,j  ,2,3)
                  matrix(i,j,4) =  gmat(i+1,j  ,1,2)
     +                            -gmat(i  ,j  ,2,5)
                  matrix(i,j,5) = -gmat(i  ,j  ,1,3)
     +                            +gmat(i  ,j+1,2,1)
     +                            -gmat(i  ,j  ,2,2)
                  matrix(i,j,6) =  gmat(i+1,j  ,1,4)
     +                            +gmat(i  ,j+1,2,5)
     +                            -gmat(i  ,j  ,2,6)
                  matrix(i,j,7) = -gmat(i  ,j  ,1,5)
     +                            +gmat(i  ,j+1,2,2)
                  matrix(i,j,8) =  gmat(i+1,j  ,1,5)
     +                            -gmat(i  ,j  ,1,6)
     +                            +gmat(i  ,j+1,2,4)
                  matrix(i,j,9) =  gmat(i+1,j  ,1,6)
     +                            +gmat(i  ,j+1,2,6)

               end do
            end do

         else if ( mprcor.eq.1 ) then

c        --- compressible different pressure correction eq.
c            See the thesis of Hester Bijl page 61
c            Mark that this approach is only allowed in case of the use of the
c            enthalpy equation used in her thesis, i.e. without diffusion and
c            without a contribution of the kinetic energy

            iprincom = iinput(34)-1
            gamcm = rinput(iprincom+1)

            if (mscal.eq.0) then
               fact = gamcm
            else if (mscal.eq.1) then
               fact = gamcm - 1d0
            end if

            vartyp = 3

c           --- interpolate enthalpy to V-1 integration points

            pnttyp = 3
            acttyp = 0
            call isintcf2 ( hnew, intp(1-nvirtual,1-nvirtual,1),
     +                      ni, nj, nvirtual, vartyp,
     +                      acttyp, pnttyp, 0, dummy )

c           --- interpolate enthalpy to V-2 integration points

            pnttyp = 2
            acttyp = 0
            call isintcf2 ( hnew, intp(1-nvirtual,1-nvirtual,2),
     +                      ni, nj, nvirtual, vartyp,
     +                      acttyp, pnttyp, 0, dummy )

            do j = 1, nj
               do i = 1, ni

                  matrix(i,j,1) =   fact*(
     +                              gmat(i+1,j  ,1,3)*intp(i+1,j,1)
     +                            - gmat(i  ,j  ,1,4)*intp(i,j,1)
     +                            + gmat(i  ,j+1,2,3)*intp(i,j+1,2)
     +                            - gmat(i  ,j  ,2,4)*intp(i,j,2) )
                  matrix(i,j,2) =   fact*(
     +                            - gmat(i  ,j  ,1,1)*intp(i,j,1)
     +                            - gmat(i  ,j  ,2,1)*intp(i,j,2) )
                  matrix(i,j,3) =   fact*(
     +                              gmat(i+1,j  ,1,1)*intp(i+1,j,1)
     +                            - gmat(i  ,j  ,1,2)*intp(i,j,1)
     +                            - gmat(i  ,j  ,2,3)*intp(i,j,2) )
                  matrix(i,j,4) =   fact*(
     +                              gmat(i+1,j  ,1,2)*intp(i+1,j,1)
     +                            - gmat(i  ,j  ,2,5)*intp(i,j,2) )
                  matrix(i,j,5) =   fact*(
     +                            - gmat(i  ,j  ,1,3)*intp(i,j,1)
     +                            + gmat(i  ,j+1,2,1)*intp(i,j+1,2)
     +                            - gmat(i  ,j  ,2,2)*intp(i,j,2) )
                  matrix(i,j,6) =   fact*(
     +                              gmat(i+1,j  ,1,4)*intp(i+1,j,1)
     +                            + gmat(i  ,j+1,2,5)*intp(i,j+1,2)
     +                            - gmat(i  ,j  ,2,6)*intp(i,j,2) )
                  matrix(i,j,7) =   fact*(
     +                            - gmat(i  ,j  ,1,5)*intp(i,j,1)
     +                            + gmat(i  ,j+1,2,2)*intp(i,j+1,2) )
                  matrix(i,j,8) =   fact*(
     +                              gmat(i+1,j  ,1,5)*intp(i+1,j,1)
     +                            - gmat(i  ,j  ,1,6)*intp(i,j,1)
     +                            + gmat(i  ,j+1,2,4)*intp(i,j+1,2) )
                  matrix(i,j,9) =   fact*(
     +                              gmat(i+1,j  ,1,6)*intp(i+1,j,1)
     +                            + gmat(i  ,j+1,2,6)*intp(i,j+1,2) )

               end do
            end do

         end if

      else if ( mcomupw.eq.1 ) then

c     --- compressible density biased upwind

         if ( mcomsup.eq.0 ) then

c        --- mcomsup=0, i.e. not the supersonic scheme

            do j = 1, nj
               do i = 1, ni

                  matrix(i,j,1) =   shiftr(i+1,j,1)*gmat(i+1,j  ,1,3)
     +                            - shiftr(i  ,j,1)*gmat(i  ,j  ,1,4)
     +                            + shiftr(i,j+1,2)*gmat(i  ,j+1,2,3)
     +                            - shiftr(i,  j,2)*gmat(i  ,j  ,2,4)

                  matrix(i,j,2) = - shiftr(i  ,j,1)*gmat(i  ,j  ,1,1)
     +                            - shiftr(i  ,j,2)*gmat(i  ,j  ,2,1)
                  matrix(i,j,3) =   shiftr(i+1,j,1)*gmat(i+1,j  ,1,1)
     +                            - shiftr(i  ,j,1)*gmat(i  ,j  ,1,2)
     +                            - shiftr(i  ,j,2)*gmat(i  ,j  ,2,3)
                  matrix(i,j,4) =   shiftr(i+1,j,1)*gmat(i+1,j  ,1,2)
     +                            - shiftr(i,  j,2)*gmat(i  ,j  ,2,5)
                  matrix(i,j,5) = - shiftr(i,  j,1)*gmat(i  ,j  ,1,3)
     +                            + shiftr(i,j+1,2)*gmat(i  ,j+1,2,1)
     +                            - shiftr(i,  j,2)*gmat(i  ,j  ,2,2)
                  matrix(i,j,6) =   shiftr(i+1,j,1)*gmat(i+1,j  ,1,4)
     +                            + shiftr(i,j+1,2)*gmat(i  ,j+1,2,5)
     +                            - shiftr(i,  j,2)*gmat(i  ,j  ,2,6)
                  matrix(i,j,7) = - shiftr(i,  j,1)*gmat(i  ,j  ,1,5)
     +                            + shiftr(i,j+1,2)*gmat(i  ,j+1,2,2)
                  matrix(i,j,8) =   shiftr(i+1,j,1)*gmat(i+1,j  ,1,5)
     +                            - shiftr(i,  j,1)*gmat(i  ,j  ,1,6)
     +                            + shiftr(i,j+1,2)*gmat(i  ,j+1,2,4)
                  matrix(i,j,9) =   shiftr(i+1,j,1)*gmat(i+1,j  ,1,6)
     +                            + shiftr(i,j+1,2)*gmat(i  ,j+1,2,6)
               end do
            end do

         else if ( mcomsup.eq.1 ) then

c        --- mcomsup=1, i.e. the supersonic scheme is used

            do j = 1, nj
               do i = 1, ni

                  matrix(i,j,1) =  (1d0-swsup(i+1,j  ,1))*
     +                                     shiftr(i+1,j,1)*
     +                                        gmat(i+1,j  ,1,3)
     +                            -(1d0-swsup(i  ,j  ,1))*
     +                                     shiftr(i  ,j,1)*
     +                                        gmat(i  ,j  ,1,4)
     +                            +(1d0-swsup(i  ,j+1,2))*
     +                                     shiftr(i,j+1,2)*
     +                                        gmat(i  ,j+1,2,3)
     +                            -(1d0-swsup(i  ,j  ,2))*
     +                                     shiftr(i,j  ,2)*
     +                                        gmat(i  ,j  ,2,4)
                  matrix(i,j,2) = -(1d0-swsup(i  ,j  ,1))*
     +                                     shiftr(i,j  ,1)*
     +                                        gmat(i  ,j  ,1,1)
     +                            -(1d0-swsup(i  ,j  ,2))*
     +                                     shiftr(i,j  ,2)*
     +                                        gmat(i  ,j  ,2,1)
                  matrix(i,j,3) =  (1d0-swsup(i+1,j  ,1))*
     +                                     shiftr(i+1,j,1)*
     +                                        gmat(i+1,j  ,1,1)
     +                            -(1d0-swsup(i  ,j  ,1))*
     +                                     shiftr(i,j  ,1)*
     +                                        gmat(i  ,j  ,1,2)
     +                            -(1d0-swsup(i  ,j  ,2))*
     +                                     shiftr(i,j  ,2)*
     +                                        gmat(i  ,j  ,2,3)
                  matrix(i,j,4) =  (1d0-swsup(i+1,j  ,1))*
     +                                     shiftr(i+1,j,1)*
     +                                        gmat(i+1,j  ,1,2)
     +                            -(1d0-swsup(i  ,j  ,2))*
     +                                     shiftr(i,j  ,2)*
     +                                        gmat(i  ,j  ,2,5)
                  matrix(i,j,5) = -(1d0-swsup(i  ,j  ,1))*
     +                                     shiftr(i,j  ,1)*
     +                                        gmat(i  ,j  ,1,3)
     +                            +(1d0-swsup(i  ,j+1,2))*
     +                                     shiftr(i,j+1,2)*
     +                                        gmat(i  ,j+1,2,1)
     +                            -(1d0-swsup(i  ,j  ,2))*
     +                                     shiftr(i,j  ,2)*
     +                                        gmat(i  ,j  ,2,2)
                  matrix(i,j,6) =  (1d0-swsup(i+1,j  ,1))*
     +                                     shiftr(i+1,j,1)*
     +                                        gmat(i+1,j  ,1,4)
     +                            +(1d0-swsup(i  ,j+1,2))*
     +                                     shiftr(i,j+1,2)*
     +                                        gmat(i  ,j+1,2,5)
     +                            -(1d0-swsup(i  ,j  ,2))*
     +                                     shiftr(i,j  ,2)*
     +                                        gmat(i  ,j  ,2,6)
                  matrix(i,j,7) = -(1d0-swsup(i  ,j  ,1))*
     +                                     shiftr(i,j  ,1)*
     +                                        gmat(i  ,j  ,1,5)
     +                            +(1d0-swsup(i  ,j+1,2))*
     +                                     shiftr(i,j+1,2)*
     +                                        gmat(i  ,j+1,2,2)
                  matrix(i,j,8) =  (1d0-swsup(i+1,j  ,1))*
     +                                     shiftr(i+1,j,1)*
     +                                        gmat(i+1,j  ,1,5)
     +                            -(1d0-swsup(i  ,j  ,1))*
     +                                     shiftr(i,j  ,1)*
     +                                        gmat(i  ,j  ,1,6)
     +                            +(1d0-swsup(i  ,j+1,2))*
     +                                     shiftr(i,j+1,2)*
     +                                        gmat(i  ,j+1,2,4)
                  matrix(i,j,9) =  (1d0-swsup(i+1,j  ,1))*
     +                                     shiftr(i+1,j,1)*
     +                                        gmat(i+1,j  ,1,6)
     +                            +(1d0-swsup(i  ,j+1,2))*
     +                                     shiftr(i,j+1,2)*
     +                                        gmat(i  ,j+1,2,6)

               end do
            end do

         end if

      end if

      if ( mcom.eq.1 ) then

c     --- mcom = 1: Compressible flow
c         Update the diagonal with appropriate terms

         iprincom = iinput(34)-1

         dt = rinput(iinput(29)-1+4)
         gamcm = rinput(iprincom+1)
         mach2 = rinput(iprincom+2)**2d0
         deltatin=1d0/dt

         cons = 1d0 + ((gamcm - 1d0)*mach2)/2d0
         cons1 = (gamcm * mach2) / cons
         if ( mequat.eq.0 ) then

c        --- mequat = 0, standard method, no mass fraction

            if ( mprcor.eq.0 ) then

c           --- mprcor = 0, standard pressure correction method

               if ( mcomupw.eq.0.and.mcomsup.eq.0 ) then

c              --- nondimensional computation

                  if ( axisym.eq.1) then

c                 --- axisymmetric case

                     do j = 1, nj
                        do i = 1, ni
                           matrix(i,j,1) = matrix(i,j,1) +
     +                      drhodp(i,j)*jac(i,j)*deltatin*
     +                      25d-2*(coor(i,j,1)+coor(i+1,j,1)
     +                      +coor(i,j+1,1)+coor(i+1,j+1,1) )
                        end do
                     end do

                  else

c                 --- Cartesian case

                     do j = 1, nj
                        do i = 1, ni
                           matrix(i,j,1) = matrix(i,j,1) +
     +                     drhodp(i,j) *jac(i,j)*deltatin
                        end do
                     end do

                  end if

               else if ( mcomupw.eq.1.or.mcomsup.eq.1 ) then

c              --- Use an upwind scheme
c                  first de i direction matrix update

                  if ( mcav.eq.0 ) then

c                 --- mcav = 0, no cavity

                     do j = 1, nj
                        do i = 1, ni

                           matrix(i,j,1) = matrix(i,j,1) +
     +                       drhodp(i,j)*deltatin*jac(i,j) +
     +                          drhodp(i,j)*densup(i,j,2,1)

                           matrix(i,j,5) = matrix(i,j,5) +
     +                       drhodp(i-1,j)*densup(i,j,1,1)

                           matrix(i,j,6) = matrix(i,j,6) +
     +                       drhodp(i+1,j)*densup(i,j,3,1)

                        end do
                     end do

                  else

c                 --- mcav = 1, cavity

                     do j = 1, nj
                        do i = 1, ni

                           matrix(i,j,1) = matrix(i,j,1) +
     +                       drhodp(i,j)*(deltatin*jac(i,j) +
     +                                       densup(i,j,2,1))
                           matrix(i,j,5) = matrix(i,j,5) +
     +                       drhodp(i-1,j)*densup(i,j,1,1)
                           matrix(i,j,6) = matrix(i,j,6) +
     +                       drhodp(i+1,j)*densup(i,j,3,1)

                        end do
                     end do

                  end if

c                 --- then the j direction update

                  if ( mcav.eq.0 ) then

c                 --- mcav = 0, no cavity

                     do j = 1, nj
                        do i = 1, ni

                           matrix(i,j,1) = matrix(i,j,1) +
     +                          drhodp(i,j)*densup(i,j,2,2)

                           matrix(i,j,3) = matrix(i,j,3) +
     +                       drhodp(i,j-1)*densup(i,j,1,2)

                           matrix(i,j,8) = matrix(i,j,8) +
     +                       drhodp(i,j+1)*densup(i,j,3,2)

                        end do
                     end do

                  else

c                 --- mcav = 1, cavity

                     do j = 1, nj
                        do i = 1, ni

                           matrix(i,j,1) = matrix(i,j,1) +
     +                       drhodp(i,j)*densup(i,j,2,2)
                           matrix(i,j,3) = matrix(i,j,3) +
     +                       drhodp(i,j-1)*densup(i,j,1,2)
                           matrix(i,j,8) = matrix(i,j,8) +
     +                       drhodp(i,j+1)*densup(i,j,3,2)

                        end do
                     end do

                  end if

               end if

            else if ( mprcor.eq.1 ) then

c           --- alternative pressure-correction, better for contacts

               call issetcon ( intp,
     +                      2*(ni+1+2*nvirtual)*(nj+1+2*nvirtual), 0d0 )

c              --- interpolate V-1 to center of cells

               vartyp = 1
               pnttyp = 4
               acttyp = 0
               call isintcf2 ( uold, intp(1-nvirtual,1-nvirtual,1),
     +                         ni, nj, nvirtual, vartyp,
     +                         acttyp, pnttyp, 0, dummy )

c              --- interpolate V-2 to center of cells

               vartyp = 2
               pnttyp = 4
               acttyp = 0
               call isintcf2 ( vold, intp(1-nvirtual,1-nvirtual,2),
     +                         ni, nj, nvirtual, vartyp,
     +                         acttyp, pnttyp, 0, dummy )

               if ( mscal.eq.0 ) then

c              --- Scaled equations

                  deltatin = 1d0/dt
                  facs=gamcm-1d0

                  do j = 1, nj
                     do i = 1, ni
                        matrix(i,j,1) = matrix(i,j,1) + cons1*deltatin*
     +                                  jac(i,j)
                     end do
                  end do

                  do j = 1, nj
                     do i = 1, ni

                        if (intp(i,j,1).gt.0d0) then
                           matrix(i,j,6)=matrix(i,j,6)-facs*cons1*
     +                                   intp(i,j,1)
                           matrix(i,j,1)=matrix(i,j,1)+facs*cons1*
     +                                   intp(i,j,1)
                        else
                           matrix(i,j,1)=matrix(i,j,1)-facs*cons1*
     +                                   intp(i,j,1)
                           matrix(i,j,5)=matrix(i,j,5)+facs*cons1*
     +                                   intp(i,j,1)
                        end if

                        if (intp(i,j,2).gt.0d0) then
                           matrix(i,j,8)=matrix(i,j,8)-facs*cons1*
     +                                   intp(i,j,2)
                           matrix(i,j,1)=matrix(i,j,1)+facs*cons1*
     +                                   intp(i,j,2)
                        else
                           matrix(i,j,1)=matrix(i,j,1)-facs*cons1*
     +                                   intp(i,j,2)
                           matrix(i,j,3)=matrix(i,j,3)+facs*cons1*
     +                                   intp(i,j,2)
                        end if

                     end do
                  end do

               else if ( mscal.eq.1 ) then

c              --- equations are dimensionfull

                  deltatin=1d0/dt
                  facs=gamcm-1d0

                  do j = 1, nj
                     do i = 1, ni
                        matrix(i,j,1) = matrix(i,j,1)+deltatin*jac(i,j)
                     end do
                  end do

                  do j = 1, nj
                     do i = 1, ni

                        if (intp(i,j,1).gt.0d0) then
                           matrix(i,j,6)=matrix(i,j,6)-facs*
     +                                   intp(i,j,1)
                           matrix(i,j,1)=matrix(i,j,1)+facs*
     +                                   intp(i,j,1)
                        else
                           matrix(i,j,1)=matrix(i,j,1)-facs*
     +                                   intp(i,j,1)
                           matrix(i,j,5)=matrix(i,j,5)+facs*
     +                                   intp(i,j,1)
                        end if

                        if (intp(i,j,2).gt.0d0) then
                           matrix(i,j,8)=matrix(i,j,8)-facs*
     +                                   intp(i,j,2)
                           matrix(i,j,1)=matrix(i,j,1)+facs*
     +                                   intp(i,j,2)
                        else
                           matrix(i,j,1)=matrix(i,j,1)-facs*
     +                                   intp(i,j,2)
                           matrix(i,j,3)=matrix(i,j,3)+facs*
     +                                   intp(i,j,2)
                        end if

                     end do
                  end do

               end if
            end if

         else if ( mequat.eq.1 ) then

c        --- mequat = 1, use mass fraction

            do i = 1, ni
               do j = 1, nj
                  matrix(i,j,1) = matrix(i,j,1) +
     +                 drhodp(i,j)*massfrac(i,j)*jac(i,j)*deltatin
               end do
            end do

         end if

      end if

      end
