      subroutine v1step  ( mat, rhs, nsmooth, sol, rbuf, ibuf)
c ======================================================================
c
c        programmer     Jos van Kan
c
c       version 1.1    10-04-1997
c         utilizes module mgpreset instead of common block cmgconst
c       version 1.0    27-03-1997
c
c
c   copyright (c) 1997
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   author.
c
c
c **********************************************************************
c
c                       DESCRIPTION
c
c   Preconditioner for mat based on a multigrid V-cycle
c   with alternating Line Jacobi as smoother.
c   Result in sol.
c **********************************************************************
c
c                       KEYWORDS
c
c   multi_grid
c   preconditioner v_cycle
c **********************************************************************
c
c         MODULES USED
c
      use mgpreset
c
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ibuf (*)
      double precision mat (*), rhs (*), sol(*), rbuf (*)
      integer nsmooth

c     ibuf      i    Integer array of dimension 300 that contains the
c                    administration of this particular multi grid block.
c     mat       i    The nx x ny x 9 coefficient matrix of the
c                    discrete Laplacian
c     nsmooth   i    Number of times smoother is applied (all levels)
c     rbuf      i    Buffer array used to store all coarse size info
c                    and as scratch space. The actaul bufferspace
c                    needed can be calculated using subr autosize.
c     rhs       i    Right hand side  (any level)
c     sol       i    Solution vector (any level)
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer ilevel, ismooth, hilevel
      double precision pi, lowb, aczero
      double precision w (100)
      double precision ChebLB (10)

c     aczero    actual zero of the shifted Chebyshev polynomial
c     cheblb    Lower bound of shifted Chebyshev interval
c               (high bound is always 2), depending on nsmooth
c               for nsmooth > 10 ChebLb = 2/3.
c     hilevel   Maximum level of coarse grids (level 0 is finest)
c     ilevel    Levelcounter (multi grid levels)
c     ismooth   Counter to number of times smoother has been applied
c     lowb      calculated lowerbound of the intervals of the shifted
c               Chebyshev polynomials (upper bound is always 2).
c     pi        the famous constant
c     w         Array of Chebyshev coefficients
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     CALCRES   Calculates residal on any level (MG)
c     ERRMSG    Reports and counts errors (MG)
c     FREESCRS  Gives reserved scratch space back to buffer (MG)
c     GETSCRS   Reserves a block of scratch space from buffer (MG)
c     LINEJACX  Calculates 1 step line jacobi preconditioner x-dir (MG)
c     LINEJACY  Calculates 1 step line jacobi preconditioner y-dir (MG)
c     PROLONGA  Prolongation (all coarsening types) (MG)
c     RESTRICT  Restriction (all coarsening types) (MG)
c     TRIADVEC  Adds x + b y with x and y vectors, b scalar (MG)
c     ZEROIZE   Initialize vector to all zeros (MG)
c **********************************************************************
c
c                       DATA STATEMENTS
c
      data ChebLB/ 0.82287565553230,  0.75375214376509,
     +             0.71927884886952,  0.70042075768052,
     +             0.68897086241745,  0.68140606981492,
     +             0.67607178474481,  0.67211862680551,
     +             0.66907476690353,  0.66665981350962 /
c ======================================================================
c
      if (ibuf (prelud_i).ne.YES) then
      call errmsg ('v1step', 'prelude has not been called previously',
     +              .true., ibuf)

c     --- fatal error

      end if
      if (nsmooth.gt.100) then
      call errmsg ('v1step', 'nsmooth too large', .true., ibuf)
      end if

c     --- calculate chebyshev coefficients

      pi = 4d0 * atan (1d0)
      if (nsmooth.le.10) then
      lowb = ChebLB (nsmooth)
      else

c     --- asymptotic value

      lowb = 2d0/3d0
      end if

      do ismooth = 1, nsmooth

c     --- calculate zero of Chebyshev polynomial

      aczero = cos (pi * (ismooth - 0.5d0)/nsmooth)
      aczero = (1d0 - 0.5 * lowb) * aczero + (1d0 + 0.5 * lowb)
      w (ismooth) = 1d0/aczero
      end do

c     --- we take the given rhs as rhs and restrict to all levels

      call restrict (rhs, rbuf (ibuf (rhsri + 1)),
     +       ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizexi+1), ibuf (sizeyi+1),
     +       rbuf, ibuf,
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))
      hilevel = ibuf (hileveli)
      do ilevel = 1, hilevel-1
        call restrict (rbuf (ibuf (rhsri + ilevel)),
     +                 rbuf (ibuf (rhsri + ilevel + 1))
     +       , ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel)
     +       , ibuf (sizexi + ilevel + 1), ibuf (sizeyi + ilevel + 1),
     +       rbuf, ibuf,
     +     0,0,0,0)
      end do

c     --- initialize solution to 0 on all levels

      call zeroize (sol, ibuf (sizexi), ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))
      do ilevel = 1, hilevel
        call zeroize (rbuf (ibuf (solri + ilevel)),
     +    ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
      end do

c--- get some space to store A x
      call getscrs ( ibuf,
     +         (ibuf (sizexi) + ibuf (slacki) + ibuf (slacki + 1))*
     +         (ibuf (sizeyi) + ibuf (slacki + 2)+ ibuf (slacki + 3)))

c     --- solution on coarsest grid

      do ismooth = 1, nsmooth
       call calcres (rbuf (ibuf (matri + hilevel)),
     +               rbuf (ibuf (solri + hilevel)),
     +               rbuf (ibuf (rhsri + hilevel)),
     +               rbuf (ibuf (resri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      rbuf (ibuf (offscri)),
     +     0,0,0,0)

       call linejacx (rbuf (ibuf (LUxri + hilevel)),
     +                rbuf (ibuf (resri + hilevel)),
     +                rbuf (ibuf (corri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +     0,0,0,0)

       call triadvec (rbuf (ibuf (solri + hilevel)), w(ismooth),
     +                rbuf (ibuf (corri + hilevel)),
     +                rbuf (ibuf (solri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +     0,0,0,0)

       call calcres (rbuf (ibuf (matri + hilevel)),
     +               rbuf (ibuf (solri + hilevel)),
     +               rbuf (ibuf (rhsri + hilevel)),
     +               rbuf (ibuf (resri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +      rbuf (ibuf (offscri)),
     +     0,0,0,0)

       call linejacy (rbuf (ibuf (LUyri + hilevel)),
     +                rbuf (ibuf (resri + hilevel)),
     +                rbuf (ibuf (corri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +     0,0,0,0)

       call triadvec (rbuf (ibuf (solri + hilevel)), w(ismooth),
     +                rbuf (ibuf (corri + hilevel)),
     +                rbuf (ibuf (solri + hilevel)),
     +      ibuf (sizexi + hilevel), ibuf (sizeyi + hilevel),
     +     0,0,0,0)
       end do

c      --- for remaining levels

      do ilevel = hilevel - 1, 1, -1

c     --- prolongate to next finer level

        call prolonga (rbuf (ibuf (solri + ilevel)),
     +                   rbuf (ibuf (solri + ilevel + 1)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +      ibuf (sizexi + ilevel+1), ibuf (sizeyi + ilevel+1),
     +      rbuf, ibuf,
     +     0,0,0,0)

c       --- smoothe solution on this level

        do ismooth = 1, nsmooth
       call calcres (rbuf ( ibuf (matri + ilevel)),
     +               rbuf ( ibuf (solri + ilevel)),
     +               rbuf ( ibuf (rhsri + ilevel)),
     +               rbuf ( ibuf (resri + ilevel)),
     +      ibuf (sizexi + ilevel),  ibuf (sizeyi + ilevel),
     +      rbuf (ibuf (offscri)),
     +     0,0,0,0)

        call linejacx (rbuf (ibuf (LUxri + ilevel)),
     +                 rbuf (ibuf (resri + ilevel)),
     +                 rbuf (ibuf (corri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)

        call triadvec (rbuf (ibuf (solri + ilevel)), w(ismooth),
     +                 rbuf (ibuf (corri + ilevel)),
     +                 rbuf (ibuf (solri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)

        call calcres (rbuf (ibuf (matri + ilevel)),
     +                rbuf (ibuf (solri + ilevel)),
     +                rbuf (ibuf (rhsri + ilevel)),
     +                rbuf (ibuf (resri + ilevel)),
     +      ibuf (sizexi + ilevel),  ibuf (sizeyi + ilevel),
     +      rbuf (ibuf (offscri)),
     +     0,0,0,0)

        call linejacy (rbuf (ibuf (LUyri + ilevel)),
     +                 rbuf (ibuf (resri + ilevel)),
     +                 rbuf (ibuf (corri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)

        call triadvec (rbuf (ibuf (solri + ilevel)), w(ismooth),
     +                 rbuf (ibuf (corri + ilevel)),
     +                 rbuf (ibuf (solri + ilevel)),
     +      ibuf (sizexi + ilevel), ibuf (sizeyi + ilevel),
     +     0,0,0,0)
        end do

      end do

c     --- prolongate to highest level

        call prolonga (sol,
     +       rbuf (ibuf (solri + 1)),
     +       ibuf (sizexi), ibuf (sizeyi),
     +       ibuf (sizexi + 1), ibuf (sizeyi + 1),
     +       rbuf, ibuf,
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

c       --- still smoothing required on highest level

      do ismooth = 1, nsmooth
      call calcres (mat, sol, rhs,
     +       rbuf(ibuf (resri)),
     +       ibuf (sizexi), ibuf (sizeyi),
     +       rbuf (ibuf (offscri)),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

      call linejacx (rbuf (ibuf (LUxri)), rbuf (ibuf (resri)),
     +       rbuf (ibuf (corri)), ibuf (sizexi), ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

      call triadvec ( sol, w(ismooth), rbuf (ibuf (corri)), sol,
     +       ibuf (sizexi),  ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

      call calcres (mat, sol, rhs, rbuf(ibuf (resri)),
     +       ibuf (sizexi), ibuf (sizeyi),
     +       rbuf (ibuf (offscri)),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

      call linejacy (rbuf (ibuf (LUyri)), rbuf (ibuf (resri)),
     +       rbuf (ibuf (corri)), ibuf (sizexi), ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))

      call triadvec ( sol, w(ismooth), rbuf (ibuf (corri)), sol,
     +       ibuf (sizexi),  ibuf (sizeyi),
     + ibuf(slacki),ibuf(slacki+1),ibuf(slacki+2),ibuf(slacki+3))
      end do

c--- return scratch space

      call freescrs (ibuf)
      end
