      subroutine isout04 ( ni, nj, nk, nvirtual, coor, vector, output )
c ======================================================================
c
c     programmer  Kees Kassels
c     version     1.2   20-01-1999 Guus: enlarge loop bounds
c     version     1.1   26-06-1997 1/sqrt(g) only if sqrt(g) >0
c     version     1.0   06-06-1996
c
c   copyright (c) 1996-1999  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the interpolation of a vector quantity
c     defined in the mid sides of the 3D cells, to the centroids
c     It is supposed that the vector contains the contravariant components
c     and that the cartesian components are required in the vertices
c     Linear interpolation (mean value) to cell centres is used,
c     Followed by linear interpolation  to vertices in the physical
c     domain (see also scalar interpolation).
c **********************************************************************
c
c                       KEYWORDS
c
c    3d
c    output
c    interpolation
c    cartesian
c    contravariant
c    transform
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nk, nvirtual
      double precision coor(1-nvirtual:ni+1+nvirtual,
     +                      1-nvirtual:nj+1+nvirtual,
     +                      1-nvirtual:nk+1+nvirtual,3),
     +                 output(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3),
     +                 vector(1-nvirtual:ni+1+nvirtual,
     +                        1-nvirtual:nj+1+nvirtual,
     +                        1-nvirtual:nk+1+nvirtual,3)

c     coor      i    reserved
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nk        i    Number of non-virtual cells in the k-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     output    o    Cartesian components of the flow field in the vertices
c     vector    i    Input flow field
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, j, k, m
      double precision a1(3), a2(3), a3(3), output1, output2, output3,
     +                 sqrtg, v1h, v1l, v2h, v2l, v3h, v3l

c     a1        Covariant xi-1 base vector
c     a2        Covariant xi-2 base vector
c     a3        Covariant xi-3 base vector
c     i         General loop variable
c     j         General loop variable
c     k         General loop variable
c     m         General loop variable
c     output1   Temporary storage for output(i,j,k,1)
c     output2   Temporary storage for output(i,j,k,2)
c     output3   Temporary storage for output(i,j,k,3)
c     sqrtg     The Jacobian in the center of a cell
c     v1h       Second flux V1 in the 1 direction
c     v1l       First  flux V1 in the 1 direction
c     v2h       Second flux V2 in the 2 direction
c     v2l       First  flux V2 in the 2 direction
c     v3h       Second flux V3 in the 3 direction
c     v3l       First  flux V3 in the 3 direction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     In order to interpolate and compute the cartesian components,
c     the contravariant components in cell centres are computed by linear
c     interpolation, these are translated to cartesian vectors and lastly
c     the cartesian vector components in the vertices are determined by
c     interpolation with isout013.
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c
c     --- Interpolation to cell centres

      do k = 0, nk+2
         do j = 0, nj+2
            do i = 0, ni+2
               v1l = vector(i,j,k,1)
               v2l = vector(i,j,k,2)
               v3l = vector(i,j,k,3)
               v1h = vector(i+1,j,k,1)
               v2h = vector(i,j+1,k,2)
               v3h = vector(i,j,k+1,3)

               output(i,j,k,1) = 0.5d0*(v1l+v1h)
               output(i,j,k,2) = 0.5d0*(v2l+v2h)
               output(i,j,k,3) = 0.5d0*(v3l+v3h)

            end do
         end do
      end do

c     --- transform to cartesian components

      do k = 0, nk+2
         do j = 0, nj+2
            do i = 0, ni+2

               output1 = output(i,j,k,1)
               output2 = output(i,j,k,2)
               output3 = output(i,j,k,3)

               do m = 1, 3

                  a1(m) = 0.25d0*
     +                     (coor(i+1,j+1,k  ,m)-coor(i,j+1,k  ,m)+
     +                      coor(i+1,j  ,k  ,m)-coor(i,j  ,k  ,m)+
     +                      coor(i+1,j  ,k+1,m)-coor(i,j  ,k+1,m)+
     +                      coor(i+1,j+1,k+1,m)-coor(i,j+1,k+1,m))

                  a2(m) = 0.25d0*
     +                     (coor(i+1,j+1,k  ,m)-coor(i+1,j,k  ,m)+
     +                      coor(i  ,j+1,k  ,m)-coor(i  ,j,k  ,m)+
     +                      coor(i  ,j+1,k+1,m)-coor(i  ,j,k+1,m)+
     +                      coor(i+1,j+1,k+1,m)-coor(i+1,j,k+1,m))

                  a3(m) = 0.25d0*
     +                     (coor(i  ,j+1,k+1,m)-coor(i  ,j+1,k,m)+
     +                      coor(i+1,j+1,k+1,m)-coor(i+1,j+1,k,m)+
     +                      coor(i+1,j  ,k+1,m)-coor(i+1,j  ,k,m)+
     +                      coor(i  ,j  ,k+1,m)-coor(i  ,j  ,k,m))

               end do

               sqrtg = a1(1)*a2(2)*a3(3)-a1(1)*a2(3)*a3(2) +
     +                 a1(2)*a2(3)*a3(1)-a1(2)*a2(1)*a3(3) +
     +                 a1(3)*a2(1)*a3(2)-a1(3)*a2(2)*a3(1)

               if ( sqrtg.gt.1d-16 ) then
                  do m = 1, 3
                     output(i,j,k,m) =
     +               ( a1(m)*output1 + a2(m)*output2 +
     +                 a3(m)*output3)/sqrtg
                  end do
               end if

            end do
         end do
      end do

      end
