      subroutine iscgstab ( nusol2, amat, rhsd, usol, res, pcgs, rbar,
     +                      qcgs, ucgs, wcgs, work, solutv, infmat,
     +                      prec, resid, initer, reiter, iinmtvc )
c ======================================================================
c
c        programmer     Kees Vuik
c        version 1.6    date 06-07-1998 Kees V: nusol, nsolut cleaned
c        version 1.5    date 21-11-1997 Kees K. iinmtvc added
c        version 1.4    date 31-10-1997 Kees V. changes for uncoupled
c        version 1.3    date 07-02-1997 Kees V. adding cgsrelax
c        version 1.2    date 21-01-1997 Kees V. adaptation of printing
c        version 1.1    date 15-01-1997 Kees V. stabilisation of omega
c        version 1.0    date 07-01-1997
c
c   copyright (c) 1994-1997  "TU Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     The Bi-CGSTAB method is used to solve an unsymmetric system of linear
c     equations of the shape A x = f.
c **********************************************************************
c
c                       KEYWORDS
c
c     linear_solver
c     preconditioner
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer          nusol2, infmat(*), initer(*), iinmtvc(*)
      double precision amat(nusol2,*), rhsd(nusol2), usol(nusol2),
     +                 res(nusol2), pcgs(nusol2), rbar(nusol2),
     +                 qcgs(nusol2), ucgs(nusol2), wcgs(nusol2),
     +                 reiter(*), prec(nusol2,*), work(nusol2,*),
     +                 solutv(*), resid(*)

c     amat      i    The coefficients matrix used in the linear system
c     iinmtvc   i    reserved
c     infmat    i    Integer information for the matrix
c     initer    i    reserved
c     nusol2    i    Number of scalar unknowns for the solvers
c                    nusol2 = nusol * ndimso
c     pcgs      i    Work array to store the direction vector in CGS.
c     prec      i    The preconditioning matrix
c     qcgs      i    Work array to store an auxiliary vector in CGS.
c     rbar      i    Work array to store the quasi-residual vector.
c     reiter    i    reserved
c     res       i    Array containing the residual vector.
c     resid     i    Array containing the norm of the residuals
c     rhsd      i    Vector containing the right-hand side vector of the
c                    system of equations.
c     solutv    i    Work array for solut with virtual points
c     ucgs      i    Work array to store an auxiliary vector in CGS.
c     usol      i    Solution vector of length n. On input the array should
c                    contain a starting vector. At output the array contains
c                    the last iterate, which is an approximation to the
c                    solution of the system.
c     wcgs      i    Work array to store an auxiliary vector in CGS.
c     work      i    Work array
c **********************************************************************
c
c                       COMMON BLOCKS
c
      include 'IScommon/cisnas'

c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      logical          start
      integer          j, isqrn, iter, istar, itmax, iprinlin, nints,
     +                 nreal, istop
      double precision rferr, sigma, alphcgs, betacgs, rhocgs, rhosta,
     +                 rnorm, ddot, dnrm2, omegcgs, epslin, eps1, eps2,
     +                 eps3, thulp, bnorm, ucgsnorm, qcgsnorm, cgsrelax

c     alphcgs   Factor alpha in the CGS-process { (rbar,r ) / (rbar,Ap) }.
c     betacgs   Factor beta in the CGS-process { (rbar,r ) / (rbar,r   ) }.
c     bnorm     Contains the norm of the righthandside vector after
c               preconditioning.
c     cgsrelax  Relaxation parameter used in Bi-CSTAB
c     eps1      Determine the accuracy of the final approximation. The
c               termination criterion is based upon
c               ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                     k                         0
c     eps2      Determine the accuracy of the final approximation. The
c               termination criterion is based upon
c               ||b-Ax || <  eps1 + eps2 * (b-Ax ) .
c                     k                         0
c     eps3      If eps3 is greater then zero, then the GMRES method
c               stops as K(A)*||b-Ax ||/||b|| < eps3.
c               An estimate of K(A) is made by the calculation of the
c               singular values of the Hessenberg matrix.
c     epslin    Required accuracy (including relative and absolute error)
c               in the linear solver.
c     iprinlin  Control parameter indicating the amount of output.
c     isqrn     The largest integer smaller than sqrt(nusol2).
c     istar     Starting number for iteration.
c     istop     Indicates which actions should be done in case of termination
c     iter      The current iteration number.
c     itmax     The maximum number of iterations to be performed in
c               the linear solver.
c     j         General loop variable
c     nints     Number of integers in a message
c     nreal     Number of reals in a message
c     omegcgs   Factor omega in the Bi-CGSTAB process
c     qcgsnorm  The norm of the vector qcgs
c     rferr     Maximal accuracy possible for this matrix, due to round-off.
c     rhocgs    Inner product of M  r and rbar at the previous iteration step
c               (where M stands for the preconditioning matrix).
c     rhosta    Rhocgs at the new iteration step.
c     rnorm     Norm of the residual vector.
c     sigma     Factor sigma in the CGS-process (inner product of rbar and
c               Ap).
c     start     Logical value indicating whether a restart is made.
c     thulp     Auxiliary variable.
c     ucgsnorm  The norm of the vector ucgs
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     DAXPY     Computes a vector + a constant * other vector. This
c               is a BLAS routine.
c     DCOPY     Copy double precision array into other one (BLAS subroutine)
c     DDOT      Computes an inner product. This is a BLAS routine.
c     DNRM2     Calculates the 2-norm of a given vector. This is
c               a BLAS routine.
c     ISDIAGMU  Computes the multiplication of a diagonal preconditioner
c               stored in array prec and a vector.
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c     ISERRCHR  Puts values of strings in character error array
c     ISERREAL  Put real value in error message
c     ISERRSUB  Error messages
c     ISMATVEC  Computes the product of the sparse matrix, stored in amat,
c               and a vector.
c     ISPREFIN  Scales the residual if a postconditioning is used
c     ISPREMAT  Computes a matrix vector product combined
c               with a pre- or postconditioning matrix
c     ISPRESTR  Scales the residual if a preconditioning is used
c     ISPRINT   ISNaS general print subroutine
c     ISWARSUB  Print warnings
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       ERROR MESSAGES
c
c    1002   work space is too small
c    1007   The required accuracy is too small.
c    1012   Method is stopped before a division by zero.
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The subroutine Bi-CGSTAB is used to solve an unsymmetric system
c     of linear equations of the shape A.x = f. To obtain a solution
c     either the Bi-CGSTAB method is used, or the preconditioned
c     Bi-CGSTAB method, possibly combined with the Eisenstat implementation.
c
c     Reference:
c
c     H.A. van der Vorst,
c     Bi-CGSTAB: A fast and smoothly converging variant of Bi-CG for the
c     solution of nonsymmetric linear systems,
c     SIAM J. Sci. Stat. Comp., 13, (1992), pp. 631-644.
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c **********************************************************************
c
c                       MODULES USED
c
c ======================================================================
c
      call iseropen ( 'iscgstab' )

c     --- Initialization of solver parameters

      iprinlin  = initer(2)
      istop     = initer(5)
      itmax     = initer(10)
      nints     = initer(19)
      eps1      = reiter(1)
      eps2      = reiter(2)
      eps3      = reiter(3)
      cgsrelax  = reiter(8)
      initer(4) = 0
      rferr     = nusol2
      rferr     = sqrt ( rferr )
      isqrn     = nint( rferr )
      rferr     = 0d0
      rhocgs    = 1d0
      istar     = 1
      betacgs   = 0d0
      alphcgs   = 1d0
      omegcgs   = 1d0
      start     = .true.

c     --- If initer(1) equals 2 (ILUD preconditioner), then the linear
c         system is scaled by a diagonal matrix, such that diag(L)=diag(U)=D=I.

      if ( initer(1).eq.2 ) then
         call isdiagmu ( nusol2, rhsd, prec(1,2) )
      end if

c     --- In order to avoid underflow in the solution of a lower
c         triangular system, we take as start solution
c         usol = usol + ||rhsd||*10^(-34)

      if ( abs(initer(1)).eq.2 .or. abs(initer(1)).eq.3 ) then
         rnorm = dnrm2 ( nusol2, rhsd, 1 )*1d-34
           if ( rnorm.gt.0 ) then
             do j = 1, nusol2
                usol(j) = usol(j)+rnorm
             end do
           end if
      end if

      rnorm  = 0
      epslin = 0
      call isprestr ( nusol2, rhsd , work(1,2), amat, infmat,
     +                prec, initer )
      bnorm = dnrm2 ( nusol2, work(1,2), 1 )

c     --- The update solution is set equal to zero.

      do j = 1, nusol2
         work(j,1) = 0d0
      end do

100   continue

c     --- Continuation after convergence, or after a restart due to
c         large rounding errors.

      call dcopy ( nusol2, work, 1, work(1,2), 1 )
      if ( initer(1).eq.2 .or. initer(1).lt.0 ) then
         call isprefin ( nusol2, work(1,2), work,
     +                   amat, infmat, prec, initer )
      end if

c     --- Solution usol is adapted.

      call daxpy ( nusol2, 1d0, work, 1, usol, 1 )

c     --- The update solution is set equal to zero.

      do j = 1, nusol2
         work(j,1) = 0d0
      end do

c     --- Compute residual vector and its norm.

      call ismatvec (amat, usol, solutv, qcgs ,infmat, initer,iinmtvc)

      if ( initer(1).eq.2 ) then
         do j = 1, nusol2
            work(j,2) = rhsd(j)-qcgs(j)-2*usol(j)
         end do
      else
         do j = 1, nusol2
            work(j,2) = rhsd(j)-qcgs(j)
         end do
      end if
      call isprestr ( nusol2, work(1,2), res, amat, infmat,
     +                prec, initer )
      rnorm = dnrm2 ( nusol2, res, 1 )

      if ( istar.eq.1 ) then
         epslin = eps1 + rnorm * eps2
         if ( eps3.gt.0 ) epslin = eps3*bnorm
         thulp  = 1000*epsmac*bnorm
         if ( epslin.lt.thulp .and. rnorm.gt.0 ) then
            if ( iprinlin.ge.1 ) then
               call iserreal ( thulp, 1 )
               call iserreal ( epslin, 2 )
               call iswarsub ( 1007, 0, 2, 0 )
            end if
            epslin = thulp
         end if
      end if

      resid(istar) = rnorm

      if ( rnorm.lt.epslin ) then
         initer(3) = 0
         goto 500
      end if

c     --- Calculation of rbar.

      call dcopy ( nusol2, res, 1, rbar, 1 )

c     --- Main iteration loop.

      do iter = istar, itmax

c     --- Check for convergence.

         if ( rnorm.lt.epslin ) then
            istar = iter
            goto 100
         end if

         initer(4) = iter

c        --- Check for maximal accuracy possible due to round-off errors.
c            If rho is smaller, then continuation is useless.

         if ( rnorm.lt.rferr .and. .not.start ) then
            istar = iter
            rferr = 0
            start = .true.
            go to 100
         end if

c        --- Compute rhosta

         call dcopy ( nusol2, res, 1, ucgs, 1 )
         rhosta = ddot ( nusol2, rbar, 1, ucgs, 1 )

c        --- In the following test restart is introduced, to test
c            only after one (or more) iterations of a restart. Otherwise
c            the test can lead to a never ending loop, when the fast
c            convergence is real and not originating from rounding
c            errors.

         if ( abs(rhosta).lt.1d-14*abs(rhocgs) .and.
     +        .not.start ) then

c        --- rhostar too small, restart process

            istar = iter
            start = .true.
            go to 100

         end if

         start = .false.

c        --- Compute the direction vector pcgs.

         if ( iter.eq.1 ) then
            call dcopy ( nusol2, ucgs, 1, pcgs, 1 )
         else
            if ( omegcgs.eq.0d0 ) then
               call iserrchr ( 'omegcgs', 1 )
               call iserrchr ( 'Bi-CGSTAB', 2 )
               if ( istop.eq.0 ) then
                  if ( iprinlin.ge.0 ) then
                     call iserrsub ( 1012, 0, 0, 2 )
                  end if
               else
                  if ( iprinlin.ge.0 ) then
                     call iswarsub ( 1012, 0, 0, 2 )
                  end if
               end if
               initer(3) = 4
               initer(4) = initer(4)-1
               goto 500
            end if
            betacgs = (rhosta/rhocgs)*(alphcgs/omegcgs)
            do j = 1, nusol2
               pcgs(j) = res(j)+betacgs*(pcgs(j)-omegcgs*wcgs(j))
            end do
         end if
         rhocgs = rhosta

c        --- Compute (preconditioned) A*pcgs, store in wcgs.

         call ispremat ( nusol2, pcgs, wcgs, amat, infmat,
     +                      prec, work(1,2), solutv, initer, iinmtvc )

c        --- Compute sigma and terminate if less than zero.

         sigma = ddot ( nusol2, rbar, 1, wcgs, 1 )

         if ( sigma.eq.0d0 ) then
            call iserrchr ( 'sigma', 1 )
            call iserrchr ( 'Bi-CGSTAB', 2 )
            if ( istop.eq.0 ) then
               if ( iprinlin.ge.0 ) then
                  call iserrsub ( 1012, 0, 0, 2 )
               end if
            else
               if ( iprinlin.ge.0 ) then
                  call iswarsub ( 1012, 0, 0, 2 )
               end if
            end if
            initer(3) = 4
            initer(4) = initer(4)-1
            goto 500
         end if

c        --- Update the vectors ucgs.

         alphcgs = rhocgs/sigma
         do j = 1, nusol2
            ucgs(j) = res(j)-alphcgs*wcgs(j)
         end do

c        --- Compute A*ucgs and store in qcgs.

         call ispremat ( nusol2, ucgs, qcgs, amat, infmat,
     +                      prec, work(1,2), solutv, initer, iinmtvc )

c        --- Compute omegcgs, the value of omegcgs is adapted in
c            order to stabilize Bi-CGSTAB. The reference for this
c            stabilization is:
c            G.L.G. Sleijpen and H.A. van der Vorst
c            Hybrid Bi-Conjugate Gradient Methods for CFD Problems
c            pp. 11-12
c            University Utrecht, Department of Mathematics,
c            Preprint 902, February 1995

         ucgsnorm = dnrm2 ( nusol2, ucgs, 1 )

         if ( ucgsnorm.eq.0d0) then
            omegcgs = 1
         else
            qcgsnorm = dnrm2 ( nusol2, qcgs, 1 )
            omegcgs = ddot ( nusol2, qcgs, 1, ucgs, 1 )/
     +                (qcgsnorm*ucgsnorm)
            omegcgs = sign(1d0,omegcgs) * max(abs(omegcgs),cgsrelax) *
     +                ucgsnorm / qcgsnorm
         end if

c        --- Update solution and residual, and compute norm.

         do j = 1, nusol2
            work(j,1) = work(j,1)+alphcgs*pcgs(j)+omegcgs*ucgs(j)
            res(j)    = ucgs(j)-omegcgs*qcgs(j)
         end do

         rnorm = dnrm2 ( nusol2, res, 1 )
         resid(iter+1) = rnorm

c        --- Adapt each sqrt(n) times rferr for accuracy check. Compute
c            the actual residual and subtract the updated residual. The
c            vector qcgs is now an indication of the possible accuracy.

         if ( mod(iter,isqrn).eq.0 ) then
            call dcopy ( nusol2, work, 1, work(1,2), 1 )
            if ( initer(1).eq.2 .or. initer(1).lt.0 ) then
               call isprefin ( nusol2, work(1,2), work,
     +                         amat, infmat, prec, initer )
            end if

c           --- Solution usol is adapted.

            call daxpy ( nusol2, 1d0, work, 1, usol, 1 )

c           --- The update solution is set equal to zero.

            do j = 1, nusol2
               work(j,1) = 0d0
            end do

c           --- Compute exact residual.

            call ismatvec ( amat, usol, solutv, qcgs ,infmat, initer,
     +                      iinmtvc )

            if ( initer(1).eq.2 ) then
               do j = 1, nusol2
                  work(j,2) = rhsd(j)-qcgs(j)-2*usol(j)
               end do
            else
               do j = 1, nusol2
                  work(j,2) = rhsd(j)-qcgs(j)
               end do
            end if
            call isprestr ( nusol2, work(1,2), qcgs, amat, infmat,
     +                      prec, initer )

c           --- Subtract exact and updated residual

            do j = 1,nusol2
               qcgs(j) = qcgs(j)-res(j)
            end do

            rferr = iter
            rferr = exp(0.5*(rferr/nusol2)**2)*dnrm2 ( nusol2, qcgs, 1 )
         end if

      end do

c     --- The iteration process terminates because the maximum number of
c         iterations is reached. Set error indicator.

      initer(3) = 3

c     --- Normal termination of the iteration process.

 500  continue

      if ( initer(3).ne.0 ) then

c     --- Update the solution.

         call dcopy ( nusol2, work, 1, work(1,2), 1 )
         if ( initer(1).eq.2 .or. initer(1).lt.0 ) then
            call isprefin ( nusol2, work(1,2), work,
     +                      amat, infmat, prec, initer )
         end if

c        --- Solution usol is adapted.

         call daxpy ( nusol2, 1d0, work, 1, usol, 1 )

      end if

c     --- If initer(1) equals 2 (ILUD preconditioner), then the linear
c         system is scaled by a diagonal matrix, such that diag(L)=diag(U)=D=I.
c         Below the right hand side is scaled back

      if ( initer(1).eq.2 ) then
         call isdiagmu ( nusol2, rhsd, prec )
      end if

c     --- Printing of linear solver information.

      if ( iprinlin.ge.1 .or. iprinlin.eq.0 .and. initer(3).ne.0 ) then

         nreal = initer(4)+1

c        --- Test if the amount of information contained in reiter
c            is less than 100 reals. Otherwise a warning is given.
c            The memory is reserved in subroutine islsds02.

         if ( initer(20).gt.100) then
            call iswarsub ( 1002, 0, 0, 0 )
         end if

         do j = 1, min ( 100, initer(20) )
            resid(nreal+j) = reiter(j)
         end do

         nreal = nreal+min ( 100, initer(20) )

         call isprint( 108, nints, initer, nreal, resid, 1, ' ')
      end if

      call iserclos ( 'iscgstab' )

      end
