      subroutine ismtvc23 ( gmat, pressure, rhside, ni, nj,
     +                      nvirtual, gmctinv, massfrac, mequat )
c ======================================================================
c
c        programmer     Guus Segal
c
c        version 2.1    date 29-10-1998 Duncan, open/close added
c        version 2.0    date 07-08-1997 Hester, extra parameters
c        version 1.0    date 03-10-1995
c
c
c   copyright (c) 1995-1997  "TU-Delft"
c   permission to copy or distribute this software or documentation
c   in hard copy or soft copy granted only by written license
c   obtained from "TU-Delft".
c   all rights reserved. no part of this publication may be reproduced,
c   stored in a retrieval system ( e.g., in memory, disk, or core)
c   or be transmitted by any means, electronic, mechanical, photocopy,
c   recording, or otherwise, without written permission from the
c   publisher.
c
c **********************************************************************
c
c                       DESCRIPTION
c
c     This subroutine performs the of the matrix vector multiplication
c     f := f - g p in 2D
c     where p is the pressure including virtual points,
c           g is the gradient matrix (no virtual points) and
c           f is the right-hand side (no virtual points) to be updated
c **********************************************************************
c
c                       KEYWORDS
c
c      multiplication
c      matrix
c      vector
c **********************************************************************
c
c                       INPUT / OUTPUT PARAMETERS
c
      implicit none
      integer ni, nj, nvirtual, gmctinv(0:3,2,6), mequat
      double precision gmat(ni+1,nj+1,2,6),
     +                 rhside(ni+1,nj+1,2),
     +                 massfrac(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual),
     +                 pressure(1-nvirtual:ni+1+nvirtual,
     +                          1-nvirtual:nj+1+nvirtual)

c     gmat      i    Gradient matrix
c     gmctinv   i    Connection+row degree of freedom to column indices
c                    for gradient matrix
c     massfrac  i    Array containing the mass fraction
c     mequat    i    Indicates the type of momentum equation to be solved:
c                    0:  standard equation
c                    1:  multi-phase gas flow
c     ni        i    Number of non-virtual cells in the i-direction (one block)
c     nj        i    Number of non-virtual cells in the j-direction (one block)
c     nvirtual  i    Number of rows of virtual cells
c     pressure  i    Array containing the pressure
c     rhside   i/o   reserved
c **********************************************************************
c
c                       COMMON BLOCKS
c
c **********************************************************************
c
c                       LOCAL PARAMETERS
c
      integer i, ishift, j, jshift, k
      double precision massmean

c     i         General loop variable
c     ishift    Shift in i-direction
c     j         General loop variable
c     jshift    Shift in j-direction
c     k         General loop variable
c     massmean  Mean value of mass fraction
c **********************************************************************
c
c                       I/O
c
c **********************************************************************
c
c                       SUBROUTINES CALLED
c
c     ISERCLOS  Resets old name of previous subroutine of higher level
c     ISEROPEN  Produces concatenated name of local subroutine
c **********************************************************************
c
c                       ERROR MESSAGES
c
c **********************************************************************
c
c                       PSEUDO CODE
c
c     The matrix vector multiplication is defined by:  a   x  = f
c                                                       ij  j    i
c
c     Since the matrix has a different structure for both degrees of freedom
c     we have to perform different loops
c
c     Algorithm:
c
c     For k := 1, 6 do
c        ishift := gmctinv(1,1,k)
c        jshift := gmctinv(2,1,k)
c        For i := 1 (1) ni+1 do
c           For j := 1 (1) nj do
c              f(i,j) := f(i,j,1)+a(i,j,1,k)*x(i+ishift,j+jshift)
c
c     For k := 1, 6 do
c        ishift := gmctinv(1,2,k)
c        jshift := gmctinv(2,2,k)
c        For i := 1 (1) ni do
c           For j := 1 (1) nj+1 do
c              f(i,j) := f(i,j,2)+a(i,j,2,k)*x(i+ishift,j+jshift)
c **********************************************************************
c
c                       MODULES USED
c
c **********************************************************************
c
c                       DATA STATEMENTS
c
c ======================================================================
c

      call iseropen('ismtvc23')

      if ( mequat.eq.0 ) then

c     --- standard pressure gradient
c         Loop over the connections

         do k = 1, 6

            ishift = gmctinv(1,1,k)
            jshift = gmctinv(2,1,k)

            do j = 1, nj
               do i = 1, ni+1
                  rhside(i,j,1) = rhside(i,j,1) - gmat(i,j,1,k) *
     +                            pressure(i+ishift,j+jshift)
               end do
            end do

            ishift = gmctinv(1,2,k)
            jshift = gmctinv(2,2,k)

            do j = 1, nj+1
               do i = 1, ni
                  rhside(i,j,2) = rhside(i,j,2) - gmat(i,j,2,k) *
     +                            pressure(i+ishift,j+jshift)
               end do
            end do

         end do

      else

c     --- for 2-phase flow: massfracaction*press gradient

         do k = 1, 6

            ishift = gmctinv(1,1,k)
            jshift = gmctinv(2,1,k)

            do j = 1, nj
               do i = 1, ni+1
                  massmean = 0.5d0*(massfrac(i,j)+massfrac(i-1,j))
                  rhside(i,j,1) = rhside(i,j,1) - gmat(i,j,1,k) *
     +                            massmean*pressure(i+ishift,j+jshift)
               end do
            end do

            ishift = gmctinv(1,2,k)
            jshift = gmctinv(2,2,k)

            do j = 1, nj+1
               do i = 1, ni
                  massmean = 0.5d0*(massfrac(i,j)+massfrac(i,j-1))
                  rhside(i,j,2) = rhside(i,j,2) - gmat(i,j,2,k) *
     +                            massmean*pressure(i+ishift,j+jshift)
               end do
            end do

         end do

      end if

      call iserclos('ismtvc23')

      end
